@@ -1,4 +1,4 @@
-__version__ = "2023.1.1"
+__version__ = "2024.1.6"
 __nabu_modules__ = [
     "app",
     "cuda",
@@ -7,9 +7,11 @@ __nabu_modules__ = [
     "misc",
     "opencl",
     "pipeline",
+    "processing",
     "preproc",
     "reconstruction",
     "resources",
     "thirdparty",
+    "stitching",
 ]
 version = __version__
