@@ -3,7 +3,10 @@
 #
 
 # Default configuration for "bootstrap" command
-from nabu.stitching.config import StitchingType
+from nabu.stitching.definitions import StitchingType
+from nabu.pipeline.config_validators import str2bool
+
+from tomoscan.framereducer.method import ReduceMethod
 
 
 BootstrapConfig = {
@@ -149,6 +152,24 @@ ReconstructConfig = {
     },
 }
 
+
+MultiCorConfig = ReconstructConfig.copy()
+MultiCorConfig.update(
+    {
+        "cor": {
+            "help": "Positions of the center of rotation. It must be a list of comma-separated scalars, or in the form start:stop:step, where start, stop and step can all be floating-point values.",
+            "default": "",
+            "mandatory": True,
+        },
+        "slice": {
+            "help": "Slice(s) indice(s) to reconstruct, in the format z1-z2. Default (empty) is the whole volume. This overwrites the configuration file start_z and end_z. You can also use --slice first, --slice last, --slice middle, and --slice all",
+            "default": "",
+            "mandatory": True,
+        },
+    }
+)
+
+
 GenerateInfoConfig = {
     "hist_file": {
         "help": "HDF5 file containing the histogram, either the reconstruction file or a dedicated histogram file.",
@@ -299,7 +320,7 @@ CompareVolumesConfig = {
 
 # Default configuration for "stitching" command
 StitchingConfig = {
-    "input_file": {
+    "input-file": {
         "help": "Nabu configuraiton file for stitching (can be obtain from nabu-stitching-boostrap command)",
         "default": "",
         "mandatory": True,
@@ -308,11 +329,16 @@ StitchingConfig = {
         "help": "Logging level. Can be 'debug', 'info', 'warning', 'error'. Default is 'info'.",
         "default": "info",
     },
+    "--only-create-master-file": {
+        "help": "Will create the master file with all sub files (volumes or scans). It expects the processing to be finished. It can happen if all slurm job have been submitted but you've been kicked out of the cluster of if you need to relaunch manually some failling job slurm for any reason",
+        "default": False,
+        "action": "store_true",
+    },
 }
 
 # Default configuration for "stitching-bootstrap" command
 BootstrapStitchingConfig = {
-    "stitching_type": {
+    "stitching-type": {
         "help": f"User can provide stitching type to filter some parameters. Must be in {StitchingType.values()}.",
         "default": None,
     },
@@ -324,6 +350,11 @@ BootstrapStitchingConfig = {
         "help": "output file to store the configuration",
         "default": "stitching.conf",
     },
+    "datasets": {
+        "help": "datasets to be stitched together",
+        "default": tuple(),
+        "nargs": "*",
+    },
 }
 
 
@@ -391,7 +422,7 @@ CompositeCorConfig = {
     "--output_file": {
         "type": str,
         "required": False,
-        "help": "Optional. Where the list of cors will be written. Default is the filename postixed with cors.txt",
+        "help": "Optional. Where the list of cors will be written. Default is the filename postixed with cors.txt. If the output filename is postfixed with .json the output will be in json format",
     },
     "--cor_options": {
         "type": str,
@@ -421,3 +452,175 @@ CreateDistortionMapHorizontallyMatchedFromPolyConfig = {
         "default": "info",
     },
 }
+
+DiagToRotConfig = {
+    "--diag_file": dict(
+        required=True, help="The reconstruction file obtained by nabu-helical using the diag_zpro_run option", type=str
+    ),
+    "--entry_name": dict(
+        required=False,
+        help="entry_name. Defauls is entry0000",
+        default="entry0000",
+    ),
+    "--near": dict(
+        required=False,
+        help="This is a relative offset respect to the center of the radios. The cor will be searched around the provided value. If not given the optinal parameter original_scan must be the original nexus file; and the estimated core will be taken there. The netry_name parameter also must be provided in this case",
+        default=None,
+        type=float,
+    ),
+    "--original_scan": dict(
+        required=False,
+        help="The original nexus file. Required only if near parameter is not given",
+        default=None,
+        type=str,
+    ),
+    "--entry_name": dict(
+        required=False,
+        help="The original nexus file entry name. Required only if near parameter is not given",
+        default=None,
+        type=str,
+    ),
+    "--near_width": dict(
+        required=False,
+        help="For the horizontal correlation, searching the cor. The radius around the near value",
+        default=20,
+        type=int,
+    ),
+    "--low_pass": dict(
+        required=False,
+        help="Data are filtered horizontally. details smaller than the provided value are filtered out. Default is 1( gaussian sigma)",
+        default=1.0,
+        type=float,
+    ),
+    "--high_pass": dict(
+        required=False,
+        help="Data are filtered horizontally. Bumps larger than the provided value are filtered out. Default is 10( gaussian sigma)",
+        default=10,
+        type=int,
+    ),
+    "--linear_interpolation": dict(
+        required=False, help="If True(default) the cor will vary linearly with z_transl", default=True, type=str2bool
+    ),
+    "--use_l1_norm": dict(
+        required=False,
+        default=True,
+        help="If false then a L2 norm will be used for the error metric, considering the overlaps, if true L1 norm will be considered",
+        type=str2bool,
+    ),
+    "--cor_file": dict(required=True, help="The file where the information to correct the cor are written", type=str),
+}
+
+DiagToPixConfig = {
+    "--diag_file": dict(
+        required=True, help="The reconstruction file obtained by nabu-helical using the diag_zpro_run option", type=str
+    ),
+    "--entry_name": dict(required=False, help="entry_name. Defauls is entry0000", default="entry0000", type=str),
+    "--search_radius_v": dict(
+        required=False,
+        help="For the vertical correlation, The maximal error in pixels of one turn respect to a contiguous one. Default is 20 ",
+        default=20,
+        type=int,
+    ),
+    "--nexus_target": dict(
+        required=False,
+        help="If given, the mentioned file will be edited with the proper pixel size, the proper COR, and corrected x_translations",
+        default=None,
+        type=str,
+    ),
+    "--nexus_source": dict(
+        required=False,
+        help="Optionaly given, used only if nexus_target has been give. The nexus file will be edited and written on nexus_target. Otherwise nexus_target is considered to be the source",
+        default=None,
+        type=str,
+    ),
+}
+
+CorrectRotConfig = {
+    "--cor_file": dict(required=True, help="The file produce by diag_to_rot", type=str),
+    "--entry_name": dict(required=False, help="entry_name. Defauls is entry0000", default="entry0000", type=str),
+    "--nexus_target": dict(
+        required=True,
+        help="The given file will be edited with the proper pixel size, the proper COR, and corrected x_translations",
+        default=None,
+        type=str,
+    ),
+    "--nexus_source": dict(
+        required=True, help="The nexus file will be edited and written on nexus_target", default=None, type=str
+    ),
+}
+
+
+ReduceDarkFlatConfig = {
+    "dataset": {"help": "Dataset (NXtomo or EDF folder) to be treated", "mandatory": True},
+    "entry": {
+        "dest": "entry",
+        "help": "an entry can be specify in case of an NXtomo",
+        "default": None,
+        "required": False,
+    },
+    "dark-method": {
+        "help": f"Define the method to be used for computing darks. Valid methods are {ReduceMethod.values()}",
+        "default": ReduceMethod.MEAN,
+        "required": False,
+    },
+    "flat-method": {
+        "help": f"Define the method to be used for computing flats. Valid methods are {ReduceMethod.values()}",
+        "default": ReduceMethod.MEDIAN,
+        "required": False,
+    },
+    "overwrite": {
+        "dest": "overwrite",
+        "action": "store_true",
+        "default": False,
+        "help": "Overwrite dark/flats if exists",
+        "required": False,
+    },
+    "debug": {
+        "dest": "debug",
+        "action": "store_true",
+        "default": False,
+        "help": "Set logging system in debug mode",
+        "required": False,
+    },
+    "output-reduced-flats-file": {
+        "aliases": ("orfl",),
+        "default": None,
+        "help": "Where to save reduced flats. If not provided will be dump near the .nx file at {scan_prefix}_flats.hdf5",
+        "required": False,
+    },
+    "output-reduced-flats-data-path": {
+        "aliases": ("output-reduced-flats-dp", "orfdp"),
+        "default": None,
+        "help": "Path in the output reduced flats file to save the dataset. If not provided will be saved at {entry}/flats/",
+        "required": False,
+    },
+    "output-reduced-darks-file": {
+        "aliases": ("ordf",),
+        "default": None,
+        "help": "Where to save reduced dark. If not provided will be dump near the .nx file at {scan_prefix}_darks.hdf5",
+        "required": False,
+    },
+    "output-reduced-darks-data-path": {
+        "aliases": ("output-reduced-darks-dp", "orddp"),
+        "default": None,
+        "help": "Path in the output reduced darks file to save the dataset. If not provided will be saved at {entry}/darks/",
+        "required": False,
+    },
+}
+
+ShowReconstructionTimingsConfig = {
+    "logfile": {
+        "help": "Path to the log file.",
+        "default": "",
+        "mandatory": True,
+    },
+    "cutoff": {
+        "help": "Cut-off parameter. Timings below this value will be discarded. For a upper-bound cutoff, provide a value in the form 'low, high'",
+        "default": None,
+    },
+    "type": {
+        "help": "How to display the result. Default is a pie chart. Possible values are: pie, bars, violin",
+        "default": "pie",
+        "type": str,
+    },
+}
