@@ -1,5 +1,4 @@
 from .. import version
-from ..resources.utils import is_hdf5_extension
 from ..pipeline.config import parse_nabu_config_file
 from ..pipeline.config_validators import convert_to_int
 from .cli_configs import ReconstructConfig
@@ -58,13 +57,7 @@ def get_log_file(arg_logfile, legacy_arg_logfile, forbidden=None):
     return logfile
 
 
-def main():
-    args = parse_params_values(
-        ReconstructConfig,
-        parser_description=f"Perform a tomographic reconstruction.",
-        program_version="nabu " + version,
-    )
-
+def get_reconstructor(args, overwrite_options=None):
     # Imports are done here, otherwise "nabu --version" takes forever
     from ..pipeline.fullfield.processconfig import ProcessConfig
     from ..pipeline.fullfield.reconstruction import FullFieldReconstructor
@@ -85,6 +78,10 @@ def main():
     logfile = get_log_file(args["logfile"], args["log_file"], forbidden=[args["input_file"]])
     conf_dict = parse_nabu_config_file(args["input_file"])
     update_reconstruction_start_end(conf_dict, args["slice"].strip())
+    if overwrite_options is not None:
+        for option_key, option_val in overwrite_options.items():
+            opt_section, opt_name = option_key.split("/")
+            conf_dict[opt_section][opt_name] = option_val
 
     proc = ProcessConfig(conf_dict=conf_dict, create_logger=logfile)
     logger = proc.logger
@@ -100,13 +97,23 @@ def main():
         "force_grouped_mode": bool(args["force_use_grouped_pipeline"]),
     }
 
-    R = FullFieldReconstructor(proc, logger=logger, extra_options=extra_options)
+    reconstructor = FullFieldReconstructor(proc, logger=logger, extra_options=extra_options)
+    return reconstructor
+
+
+def main():
+    args = parse_params_values(
+        ReconstructConfig,
+        parser_description=f"Perform a tomographic reconstruction.",
+        program_version="nabu " + version,
+    )
+
+    R = get_reconstructor(args)
+    proc = R.process_config
+
     R.reconstruct()
-    R.merge_data_dumps()
-    if is_hdf5_extension(proc.nabu_config["output"]["file_format"]):
-        R.merge_hdf5_reconstructions()
-    else:
-        R.merge_histograms()
+    R.finalize_files_saving()
+    return 0
 
 
 if __name__ == "__main__":
