@@ -7,8 +7,8 @@ try:
     import pycuda
     import pycuda.driver as cuda
     from pycuda import gpuarray as garray
-    from pycuda.tools import make_default_context
     from pycuda.tools import clear_context_caches
+    from pycuda.compiler import get_nvcc_version as pycuda_get_nvcc_version
 
     __has_pycuda__ = True
     __pycuda_error_msg__ = None
@@ -25,6 +25,13 @@ try:
 except ImportError:
     __has_cufft__ = False
 
+try:
+    import cupy
+
+    __has_cupy__ = True
+except ImportError:
+    __has_cupy__ = False
+
 
 def get_cuda_context(device_id=None, cleanup_at_exit=True):
     """
@@ -134,6 +141,27 @@ def collect_cuda_gpus():
     return cuda_gpus
 
 
+def get_nvcc_version(nvcc_cmd="nvcc"):
+    try:
+        ver = "".join(pycuda_get_nvcc_version(nvcc_cmd)).split("release")[1].strip().split(" ")[0].strip(",")
+    except:
+        ver = None
+    return ver
+
+
+def check_textures_availability():
+    """
+    Check whether Cuda textures can be used.
+    The only limitation is pycuda which does not support texture objects.
+    Textures references were deprecated, and removed from Cuda 12.
+    """
+    nvcc_ver = get_nvcc_version()
+    if nvcc_ver is None:
+        return False  # unknown - can't parse NVCC version for some reason
+    nvcc_major = int(nvcc_ver.split(".")[0])
+    return nvcc_major < 12
+
+
 """
 pycuda/driver.py
     np.complex64: SIGNED_INT32, num_channels = 2
@@ -183,7 +211,7 @@ def get_shape_dtype(arr):
         raise ValueError("Unknown array type %s" % str(type(arr)))
 
 
-def copy_array(dst, src, check=False, src_dtype=None):
+def copy_array(dst, src, check=False, src_dtype=None, dst_x_in_bytes=0, dst_y=0):
     """
     Copy a source array to a destination array.
     Source and destination can be either numpy.ndarray, pycuda.Driver.Array,
@@ -226,6 +254,9 @@ def copy_array(dst, src, check=False, src_dtype=None):
 
     if isinstance(dst, cuda.Array):
         copy.set_dst_array(dst)
+        # Support offset (x, y) in target (for copying to texture)
+        copy.dst_x_in_bytes = dst_x_in_bytes
+        copy.dst_y = dst_y
     elif isinstance(dst, garray.GPUArray):
         copy.set_dst_device(dst.gpudata)
     else:  # numpy
@@ -288,3 +319,13 @@ def replace_array_memory(arr, new_shape):
     arr.shape = new_shape
     # TODO re-compute strides
     return arr
+
+
+def pycuda_to_cupy(arr_pycuda):
+    arr_cupy_mem = cupy.cuda.UnownedMemory(arr_pycuda.ptr, arr_pycuda.size, arr_pycuda)
+    arr_cupy_memptr = cupy.cuda.MemoryPointer(arr_cupy_mem, offset=0)
+    return cupy.ndarray(arr_pycuda.shape, dtype=arr_pycuda.dtype, memptr=arr_cupy_memptr)  # pylint: disable=E1123
+
+
+def cupy_to_pycuda(arr_cupy):
+    return garray.empty(arr_cupy.shape, arr_cupy.dtype, gpudata=arr_cupy.data.ptr)
