@@ -5,11 +5,14 @@ import scipy.ndimage
 import h5py
 from nabu.testutils import utilstest, __do_long_tests__
 from nabu.testutils import get_data as nabu_get_data
+
 from nabu.estimation.cor import (
     CenterOfRotation,
     CenterOfRotationAdaptiveSearch,
     CenterOfRotationGrowingWindow,
     CenterOfRotationSlidingWindow,
+    CenterOfRotationFourierAngles,
+    CenterOfRotationOctaveAccurate,
 )
 from nabu.estimation.cor_sino import SinoCor
 
@@ -30,6 +33,27 @@ def bootstrap_cor_win(request):
     cls.data_ha_sino, cls.cor_ha_sn_pix = get_cor_win_sino_data_h5("halftomo_1_sino.npz")
 
 
+@pytest.fixture(scope="class")
+def bootstrap_cor_accurate(request):
+    cls = request.cls
+    cls.abs_tol = 0.2
+    cls.image_pair_stylo, cls.cor_pos_abs_stylo = get_cor_win_proj_data_h5("stylo_accurate.npz")
+    cls.image_pair_blc12781, cls.cor_pos_abs_blc12781 = get_cor_win_proj_data_h5("blc12781_accurate.npz")
+
+
+@pytest.fixture(scope="class")
+def bootstrap_cor_fourier(request):
+    cls = request.cls
+    cls.abs_tol = 0.2
+    dataset_relpath = os.path.join("sino_bamboo_hercules_for_test.npz")
+    dataset_downloaded_path = utilstest.getfile(dataset_relpath)
+    a = np.load(dataset_downloaded_path)
+    cls.sinos = a["sinos"]
+    cls.angles = a["angles"]
+    cls.true_cor = a["true_cor"]
+    cls.estimated_cor_from_motor = a["estimated_cor_from_motor"]
+
+
 def get_cor_data_h5(*dataset_path):
     """
     Get a dataset file from silx.org/pub/nabu/data
@@ -241,6 +265,7 @@ class TestCor:
         assert "Images need to be of the same shape" in str(ex.value), message
 
 
+@pytest.mark.skipif(not (__do_long_tests__), reason="Need NABU_LONG_TESTS=1 for this test")
 @pytest.mark.usefixtures("bootstrap_cor", "bootstrap_cor_win")
 class TestCorWindowSlide:
     def test_proj_center_axis_lft(self):
@@ -309,6 +334,7 @@ class TestCorWindowSlide:
         assert np.isclose(self.cor_ha_sn_pix, cor_position, atol=self.abs_tol * 5), message
 
 
+@pytest.mark.skipif(not (__do_long_tests__), reason="need NABU_LONG_TESTS for this test")
 @pytest.mark.usefixtures("bootstrap_cor", "bootstrap_cor_win")
 class TestCorWindowGrow:
     def test_proj_center_axis_cen(self):
@@ -379,12 +405,68 @@ class TestCoarseToFineSinoCor:
         Test nabu.estimation.cor_sino.SinoCor
         """
         sino_halftomo = np.vstack([self.data_ha_sino[0], self.data_ha_sino[1]])
-        sino_cor = SinoCor(sino_halftomo)
+        sino_cor = SinoCor(self.data_ha_sino[0], np.fliplr(self.data_ha_sino[1]))
         cor_coarse = sino_cor.estimate_cor_coarse()
-        assert np.isscalar(cor_coarse), f"cor_position expected to be a scale, {type(cor_position)} returned"
+        assert np.isscalar(cor_coarse), f"cor_position expected to be a scalar, {type(cor_coarse)} returned"
         cor_fine = sino_cor.estimate_cor_fine()
-        assert np.isscalar(cor_fine), f"cor_position expected to be a scale, {type(cor_position)} returned"
+        assert np.isscalar(cor_fine), f"cor_position expected to be a scale, {type(cor_fine)} returned"
 
         cor_ref = self.cor_ha_sn_pix + sino_halftomo.shape[-1] / 2.0
         message = "Computed CoR %f " % cor_fine + " and expected CoR %f do not coincide" % cor_ref
         assert abs(cor_fine - cor_ref) < self.abs_tol * 2, message
+
+
+@pytest.mark.usefixtures("bootstrap_cor_accurate")
+class TestCorOctaveAccurate:
+    def test_cor_accurate_positive_shift(self):
+        detector_width = self.image_pair_stylo[0].shape[1]
+        CoR_calc = CenterOfRotationOctaveAccurate()
+        cor_position = CoR_calc.find_shift(self.image_pair_stylo[0], np.fliplr(self.image_pair_stylo[1]), "center")
+        cor_position = cor_position + detector_width / 2
+        assert np.isscalar(cor_position), f"cor_position expected to be a scalar, {type(cor_position)} returned"
+        message = f"Computed CoR {cor_position} and expected CoR {self.cor_pos_abs_stylo} do not coincide."
+        assert np.isclose(self.cor_pos_abs_stylo, cor_position, atol=self.abs_tol), message
+
+    def test_cor_accurate_negative_shift(self):
+        detector_width = self.image_pair_blc12781[0].shape[1]
+        CoR_calc = CenterOfRotationOctaveAccurate()
+        cor_position = CoR_calc.find_shift(
+            self.image_pair_blc12781[0], np.fliplr(self.image_pair_blc12781[1]), "center"
+        )
+        cor_position = cor_position + detector_width / 2
+        assert np.isscalar(cor_position), f"cor_position expected to be a scalar, {type(cor_position)} returned"
+        message = f"Computed CoR {cor_position} and expected CoR {self.cor_pos_abs_blc12781} do not coincide."
+        assert np.isclose(self.cor_pos_abs_blc12781, cor_position, atol=self.abs_tol), message
+
+
+@pytest.mark.usefixtures("bootstrap_cor_fourier", "bootstrap_cor_win")
+class TestCorFourierAngle:
+    def test_sino_right_axis_with_near_pos(self):
+        sino1 = self.data_ha_sino[0, :, :]
+        sino2 = np.fliplr(self.data_ha_sino[1, :, :])
+        start_angle = np.pi / 4
+        angles = np.linspace(start_angle, start_angle + 2 * np.pi, 2 * sino1.shape[0])
+
+        cor_options = {"side": 740, "refine": True}
+
+        CoR_calc = CenterOfRotationFourierAngles(cor_options=cor_options)
+        cor_position = CoR_calc.find_shift(sino1, sino2, angles, side="right")
+        assert np.isscalar(cor_position), f"cor_position expected to be a scale, {type(cor_position)} returned"
+
+        message = "Computed CoR %f " % cor_position + " and expected CoR %f do not coincide" % self.cor_ha_sn_pix
+        assert np.isclose(self.cor_ha_sn_pix, cor_position, atol=self.abs_tol * 3), message
+
+    def test_sino_right_axis_with_ignore(self):
+        sino1 = self.data_ha_sino[0, :, :]
+        sino2 = np.fliplr(self.data_ha_sino[1, :, :])
+        start_angle = np.pi / 4
+        angles = np.linspace(start_angle, start_angle + 2 * np.pi, 2 * sino1.shape[0])
+
+        cor_options = {"side": "ignore", "refine": True}
+
+        CoR_calc = CenterOfRotationFourierAngles(cor_options=cor_options)
+        cor_position = CoR_calc.find_shift(sino1, sino2, angles, side="right")
+        assert np.isscalar(cor_position), f"cor_position expected to be a scale, {type(cor_position)} returned"
+
+        message = "Computed CoR %f " % cor_position + " and expected CoR %f do not coincide" % self.cor_ha_sn_pix
+        assert np.isclose(self.cor_ha_sn_pix, cor_position, atol=self.abs_tol * 3), message
