@@ -1,6 +1,6 @@
 import numpy as np
 from numpy.polynomial.polynomial import Polynomial, polyval
-from .alignment import median_filter, plt
+from .alignment import medfilt2d, plt
 from .cor import CenterOfRotation
 
 try:
@@ -139,7 +139,7 @@ class CameraTilt(CenterOfRotation):
         fitted_shifts_h += fh[1, :]
 
         # Computing tilt
-        fitted_shifts_h = median_filter(fitted_shifts_h, size=3)
+        fitted_shifts_h = medfilt2d(fitted_shifts_h, kernel_size=3)
 
         half_img_size = (img_shape[-2] - 1) / 2
         cc_v_coords = np.linspace(-half_img_size, half_img_size, img_shape[-2])
