@@ -74,7 +74,7 @@ def get_default_output_volume(
                 volume_basename=input_volume.get_volume_basename(),
             )
         else:
-            raise NotImplementedError
+            raise NotImplementedError(f"output volume format {output_type} is not handled")
     elif isinstance(input_volume, (HDF5Volume, MultiTIFFVolume)):
         if output_type == "hdf5":
             data_file_parent_path, data_file_name = os.path.split(input_volume.data_url.file_path())
@@ -121,9 +121,9 @@ def get_default_output_volume(
                 )
             )
         else:
-            raise NotImplementedError
+            raise NotImplementedError(f"output volume format {output_type} is not handled")
     else:
-        raise NotImplementedError
+        raise NotImplementedError(f"input volume format {input_volume} is not handled")
 
 
 def cast_volume(
@@ -157,6 +157,9 @@ def cast_volume(
         the created will volume will not be saved in this processing. If you want
         to save the cast volume you must do it yourself.
 
+    .. note::
+        if you want to tune compression ratios (for jp2k) then please update the `cratios` attributes of the output volume
+
     """
     if not isinstance(input_volume, VolumeBase):
         raise TypeError(f"input_volume is expected to be a {VolumeBase}. {type(input_volume)} provided")
@@ -164,6 +167,12 @@ def cast_volume(
     if not isinstance(output_volume, VolumeBase):
         raise TypeError(f"output_volume is expected to be a {VolumeBase}. {type(output_volume)} provided")
 
+    try:
+        output_data_type = numpy.dtype(
+            output_data_type
+        )  # User friendly API in case user provides np.uint16 e.g. (see issue #482)
+    except Exception:
+        pass
     if not isinstance(output_data_type, numpy.dtype):
         raise TypeError(f"output_data_type is expected to be a {numpy.dtype}. {type(output_data_type)} provided")
 
@@ -284,25 +293,48 @@ def find_histogram(volume: VolumeBase, scan: Optional[TomoScanBase] = None) -> O
             data_path = volume.data_url.data_path().replace(HDF5Volume.DATA_DATASET_NAME, "histogram/results/data")
     elif isinstance(volume, (EDFVolume, JP2KVolume, TIFFVolume, MultiTIFFVolume)):
         if isinstance(volume, (EDFVolume, JP2KVolume, TIFFVolume)):
-            # TODO: check with pierre what is the policy of histogram files names
             histogram_file = os.path.join(
                 volume.data_url.file_path(),
-                volume.get_volume_basename() + "histogram.hdf5",
+                volume.get_volume_basename() + "_histogram.hdf5",
             )
+            if not os.path.exists(histogram_file):
+                # legacy location
+                legacy_histogram_file = os.path.join(
+                    volume.data_url.file_path(),
+                    volume.get_volume_basename() + "histogram.hdf5",
+                )
+                if os.path.exists(legacy_histogram_file):
+                    # only overwrite if exists. Else keep the older one to get a clearer information
+                    histogram_file = legacy_histogram_file
         else:
-            # TODO: check with pierre what is the policy of histogram files names
             file_path, _ = os.path.splitext(volume.data_url.file_path())
-            histogram_file = os.path.join(file_path + "histogram.hdf5")
+            histogram_file = file_path + "_histogram.hdf5"
 
         if scan is not None:
-            data_path = getattr(scan, "entry", "entry")
+            data_path = getattr(scan, "entry/histogram/results/data", "entry/histogram/results/data")
         else:
-            # TODO: FIXME: how to get the entry name in every case ?
-            # possible solutions are:
-            #     * look at the different entries and check for histogram: will work if only one histogram in the file
-            #     * Add a histogram request so the user can provide it (can be done at tomoscan level or nabu if we think this is specific to nabu)
-            _logger.info("histogram file found but unable to find relevant histogram")
-            return None
+
+            def get_file_entries(file_path: str) -> Optional[tuple]:
+                if os.path.exists(file_path):
+                    with HDF5File(file_path, mode="r") as h5s:
+                        return tuple(h5s.keys())
+                else:
+                    return None
+
+            # in the case we only know about the volume to cast.
+            # in most of the cast the histogram.hdf5 file will only get a single entry. The exception could be
+            # for HDF5 if the user save volumes into the same file.
+            # we can find back the histogram
+            entries = get_file_entries(histogram_file)
+            if entries is not None and len(entries) == 1:
+                data_path = "/".join((entries[0], "histogram/results/data"))
+            else:
+                # TODO: FIXME: how to get the entry name in every case ?
+                # what to do if the histogram file has more than one entry.
+                # one option could be to request the entry from the user...
+                # or keep as today (in this case it will be recomputed)
+                _logger.info("histogram file found but unable to find relevant histogram")
+                return None
     else:
         raise NotImplementedError(f"volume {type(volume)} not handled")
 
@@ -323,7 +355,7 @@ def find_histogram(volume: VolumeBase, scan: Optional[TomoScanBase] = None) -> O
             )
 
 
-def _get_hst_saturations(hist, bins, rescale_min_percentile, rescale_max_percentile):
+def _get_hst_saturations(hist, bins, rescale_min_percentile: numpy.float32, rescale_max_percentile: numpy.float32):
     hist_cum = numpy.cumsum(hist)
     bin_index_min = numpy.searchsorted(hist_cum, numpy.percentile(hist_cum, rescale_min_percentile))
     bin_index_max = numpy.searchsorted(hist_cum, numpy.percentile(hist_cum, rescale_max_percentile))
@@ -338,14 +370,25 @@ def _try_to_find_min_max_from_histo(
     rescaling a volume
     """
     histogram_res_url = find_histogram(input_volume, scan=scan)
-    data_min = data_max = None
     if histogram_res_url is not None:
-        try:
-            histogram = get_data(histogram_res_url)
-        except Exception as e:
-            _logger.error(f"Fail to load histogram from {histogram_res_url}. Reason is {e}")
-        else:
-            bins = histogram[1]
-            hist = histogram[0]
-            data_min, data_max = _get_hst_saturations(hist, bins, rescale_min_percentile, rescale_max_percentile)
-    return data_min, data_max
+        return _min_max_from_histo(
+            url=histogram_res_url,
+            rescale_min_percentile=rescale_min_percentile,
+            rescale_max_percentile=rescale_max_percentile,
+        )
+    else:
+        return None, None
+
+
+def _min_max_from_histo(url: DataUrl, rescale_min_percentile: int, rescale_max_percentile: int) -> tuple:
+    try:
+        histogram = get_data(url)
+    except Exception as e:
+        _logger.error(f"Fail to load histogram from {url.path()}. Reason is {e}")
+        return None, None
+    else:
+        bins = histogram[1]
+        hist = histogram[0]
+        return _get_hst_saturations(
+            hist, bins, numpy.float32(rescale_min_percentile), numpy.float32(rescale_max_percentile)
+        )
