@@ -9,9 +9,9 @@ from ..misc.binning import binning as image_binning
 from ..utils import subsample_dict, get_3D_subregion, get_num_threads
 
 try:
-    from silx.third_party.EdfFile import EdfFile
+    from fabio.edfimage import EdfImage
 except ImportError:
-    EdfFile = None
+    EdfImage = None
 
 
 class Reader:
@@ -122,21 +122,21 @@ class EDFReader(Reader):
         A class for reading series of EDF Files.
         Multi-frames EDF are not supported.
         """
-        if EdfFile is None:
-            raise ImportError("Need EdfFile to use this reader")
+        if EdfImage is None:
+            raise ImportError("Need fabio to use this reader")
         super().__init__(sub_region=sub_region)
+        self._reader = EdfImage()
+        self._first_fname = None
 
     def read(self, fname):
-        E = EdfFile(fname, "r")
-        if self.sub_region is not None:
-            data = E.GetData(
-                0,
-                Pos=(self.start_x, self.start_y),
-                Size=(self.end_x - self.start_x, self.end_y - self.start_y),
-            )
+        if self._first_fname is None:
+            self._first_fname = fname
+            self._reader.read(fname)
+        if self.sub_region is None:
+            data = self._reader.data
         else:
-            data = E.GetData(0)
-        E.File.close()
+            data = self._reader.fast_read_roi(fname, (slice(self.start_y, self.end_y), slice(self.start_x, self.end_x)))
+        self._reader.close()
         return data
 
     def get_data(self, data_url):
@@ -279,9 +279,10 @@ class ChunkReader:
             Whether to bin the data. If multi-dimensional binning is done,
             the parameter must be in the form (binning_x, binning_y).
             Each image will be binned by these factors.
-        dataset_subsampling: int, optional
-            Whether to subsample the dataset. If an integer `n` is provided,
-            then one image out of `n` will be read.
+        dataset_subsampling: int or tuple, optional
+            Subsampling factor when reading the images.
+            If an integer `n` is provided, then one image out of `n` will be read.
+            If a tuple of integers (step, begin) is given, the data is read as data[begin::step]
         num_threads: int, optional
             Number of threads to use for binning the data.
             Default is to use all available threads.
@@ -328,6 +329,10 @@ class ChunkReader:
     def _set_files(self, files):
         if len(files) == 0:
             raise ValueError("Expected at least one data file")
+        self._files_begin_idx = 0
+        if isinstance(self.dataset_subsampling, (tuple, list)):
+            self._files_begin_idx = self.dataset_subsampling[1]
+            self.dataset_subsampling = self.dataset_subsampling[0]
         self.n_files = len(files)
         self.files = files
         self._sorted_files_indices = sorted(files.keys())
@@ -469,10 +474,13 @@ class ChunkReader:
             self._fileindex_to_idx[fileidx] = i
 
     def _load_multi(self):
-        urls_compacted = get_compacted_dataslices(self.files, subsampling=self.dataset_subsampling)
+        urls_compacted = get_compacted_dataslices(
+            self.files, subsampling=self.dataset_subsampling, begin=self._files_begin_idx
+        )
         loaded = {}
         start_idx = 0
-        for idx in self._sorted_files_indices:
+        sorted_files_indices = sorted(urls_compacted.keys())
+        for idx in sorted_files_indices:
             url = urls_compacted[idx]
             url_str = str(url)
             is_loaded = loaded.get(url_str, False)
