@@ -1,4 +1,5 @@
 from glob import glob
+from pathlib import Path as pathlib_Path
 from os import path, getcwd, chdir
 from posixpath import join as posix_join
 from datetime import datetime
@@ -320,6 +321,70 @@ class NXProcessWriter(Writer):
             return results_path
 
 
+class NXVolVolume(NXProcessWriter):
+    """
+    An interface to NXProcessWriter with the same API than tomoscan.esrf.volume.
+
+    NX files are written in two ways:
+
+       1. Partial files containing sub-volumes
+       2. Final volume: master file with virtual dataset pointing to partial files
+
+    This class handles the first one, therefore expects the "start_index" parameter.
+    In the case of HDF5, a sub-directory is creating to contain the partial files.
+    In other words, if file_prefix="recons" and output_dir="/path/to/out":
+
+    /path/to/out/recons.h5 # final master file
+    /path/to/out/recons/
+      /path/to/out/recons/recons_00000.h5
+      /path/to/out/recons/recons_00100.h5
+      ...
+    """
+
+    def __init__(self, **kwargs):
+        # get parameters from kwargs passed to tomoscan XXVolume()
+        folder = output_dir = kwargs.get("folder", None)
+        volume_basename = file_prefix = kwargs.get("volume_basename", None)
+        start_index = kwargs.get("start_index", None)
+        overwrite = kwargs.get("overwrite", False)
+        data_path = entry = kwargs.get("data_path", None)
+        self._process_name = kwargs.get("process_name", "reconstruction")
+        if any([param is None for param in [folder, volume_basename, start_index, entry]]):
+            raise ValueError("Need the following parameters: folder, volume_basename, start_index, data_path")
+        #
+
+        # By default, a sub-folder is created so that partial volumes will be one folder below the master file
+        # (see example above in class documentation)
+        if kwargs.get("create_subfolder", True):
+            output_dir = path.join(output_dir, file_prefix)
+
+        if path.exists(output_dir):
+            if not (path.isdir(output_dir)):
+                raise ValueError("Unable to create directory %s: already exists and is not a directory" % output_dir)
+        else:
+            pathlib_Path(output_dir).mkdir(parents=True, exist_ok=True)
+        #
+
+        file_prefix += str("_%05d" % start_index)
+        fname = path.join(output_dir, file_prefix + ".hdf5")
+
+        super().__init__(fname, entry=entry, filemode="a", overwrite=overwrite)
+        self.data = None
+        self.metadata = None
+        self.file_path = fname
+
+    def save(self):
+        if self.data is None:
+            raise ValueError("Must set data first")
+        self.write(self.data, self._process_name, config=self.metadata)
+
+    def save_metadata(self):
+        pass  # already done
+
+    def browse_data_files(self):
+        return [self.fname]
+
+
 # COMPAT.
 LegacyNXProcessWriter = NXProcessWriter
 #
@@ -354,7 +419,7 @@ def export_dict_to_h5(dic, h5file, h5path, overwrite_data=True, mode="a"):
     return dicttoh5(modified_dic, h5file=h5file, h5path=h5path, update_mode=update_mode, mode=mode)
 
 
-def create_virtual_layout(files_or_pattern, h5_path, base_dir=None, axis=0):
+def create_virtual_layout(files_or_pattern, h5_path, base_dir=None, axis=0, dtype="f"):
     """
     Create a HDF5 virtual layout.
 
@@ -392,7 +457,7 @@ def create_virtual_layout(files_or_pattern, h5_path, base_dir=None, axis=0):
         shapes.append(shape)
     total_shape = merged_shape(shapes, axis=axis)
 
-    virtual_layout = VirtualLayout(shape=total_shape, dtype="f")
+    virtual_layout = VirtualLayout(shape=total_shape, dtype=dtype)
     start_idx = 0
     for vsource, shape in zip(virtual_sources, shapes):
         n_imgs = shape[axis]
@@ -426,6 +491,7 @@ def merge_hdf5_files(
     base_dir=None,
     axis=0,
     overwrite=False,
+    dtype="f",
 ):
     """
     Parameters
@@ -458,7 +524,7 @@ def merge_hdf5_files(
     """
     if base_dir is not None:
         prev_cwd = getcwd()
-    virtual_layout = create_virtual_layout(files_or_pattern, h5_path, base_dir=base_dir, axis=axis)
+    virtual_layout = create_virtual_layout(files_or_pattern, h5_path, base_dir=base_dir, axis=axis, dtype=dtype)
     nx_file = NXProcessWriter(output_file, entry=output_entry, filemode=output_filemode, overwrite=overwrite)
     nx_file.write(
         virtual_layout,
@@ -729,15 +795,15 @@ class HSTVolWriter(Writer):
         super().__init__(fname)
         self.append = append
         self._vol_writer = RawVolume(fname, overwrite=True, append=append)
+        self._hst_metadata = kwargs.get("hst_metadata", {})
 
-    @staticmethod
-    def generate_metadata(data, **kwargs):
+    def generate_metadata(self, data, **kwargs):
         n_z, n_y, n_x = data.shape
         metadata = {
             "NUM_X": n_x,
             "NUM_Y": n_y,
             "NUM_Z": n_z,
-            "voxelSize": kwargs.get("voxelSize", 40.0),
+            "voxelSize": 40.0,
             "BYTEORDER": "LOWBYTEFIRST",
             "ValMin": kwargs.get("ValMin", 0.0),
             "ValMax": kwargs.get("ValMin", 1.0),
@@ -746,6 +812,8 @@ class HSTVolWriter(Writer):
             "S1": 0.0,
             "S2": 0.0,
         }
+        for key, default_val in metadata.items():
+            metadata[key] = kwargs.get(key, None) or self._hst_metadata.get(key, None) or default_val
         return metadata
 
     @staticmethod
@@ -786,12 +854,13 @@ class HSTVolVolume(HSTVolWriter):
     """
 
     def __init__(self, **kwargs):
-        file_path = kwargs.pop("file_path", None)
+        file_path = kwargs.get("file_path", None)
         if file_path is None:
             raise ValueError("Missing mandatory 'file_path' parameter")
         super().__init__(file_path, append=kwargs.pop("append", False), **kwargs)
         self.data = None
         self.metadata = None
+        self.data_url = self._vol_writer.data_url
 
     def save(self):
         if self.data is None:
@@ -801,6 +870,9 @@ class HSTVolVolume(HSTVolWriter):
     def save_metadata(self):
         pass  # already done for HST part - proper metadata is not supported
 
+    def browse_data_files(self):
+        return [self.fname]
+
 
 # Unused - kept for compat.
 Writers = {
