@@ -1,7 +1,7 @@
 import os
 
 path = os.path
-from ..utils import is_writeable
+from ..utils import check_supported, is_writeable
 from .params import *
 
 """
@@ -63,15 +63,32 @@ def convert_to_bool(val):
     if not error:
         res = val_int > 0
     else:
-        if val.lower() in ["yes", "true"]:
+        if val.lower() in ["yes", "true", "y"]:
             res = True
             error = None
-        if val.lower() in ["no", "false"]:
+        if val.lower() in ["no", "false", "n"]:
             res = False
             error = None
     return res, error
 
 
+def str2bool(val):
+    """This is an interface to convert_to_bool and it is meant
+    to work as a class: in argparse interface the type argument can be set to float, int .. in general to a class.
+    The argument value is then created, at parsing time, by typecasting the input string to the given class.
+    A possibly occuring exception then trigger, in case, the display explanation provided by the argparse library.
+    All what this methods does is simply trying to convert an argument into a bool, and return it,
+    or generate an exception if there is a problem
+    """
+    import argparse
+
+    res, error = convert_to_bool(val)
+    if error:
+        raise argparse.ArgumentTypeError(error)
+    else:
+        return res
+
+
 def convert_to_bool_noerr(val):
     res, err = convert_to_bool(val)
     if err is not None:
@@ -226,6 +243,20 @@ def binning_validator(val):
     return max(1, val_int)
 
 
+@validator
+def projections_subsampling_validator(val):
+    val = val.strip()
+    err_msg = "projections_subsampling: expected one positive integer or two integers in the format step:begin"
+    if ":" not in val:
+        val += ":0"
+    step, begin = val.split(":")
+    step_int, error1 = convert_to_int(step)
+    begin_int, error2 = convert_to_int(begin)
+    if error1 is not None or error2 is not None or step_int <= 0 or begin_int < 0:
+        raise ValueError(err_msg)
+    return step_int, begin_int
+
+
 @validator
 def optional_file_name_validator(val):
     if len(val) > 0:
@@ -524,6 +555,35 @@ def logging_validator(val):
     return name_range_checker(val, set(log_levels.values()), "logging level", replacements=log_levels)
 
 
+@validator
+def exclude_projections_validator(val):
+    val = val.strip()
+    if val == "":
+        return None
+    if path.isfile(val):
+        # previous/default behavior
+        return {"type": "indices", "file": val}
+    if "=" not in val:
+        raise ValueError(
+            "exclude_projections: expected either 'angles=angles_file.txt' or 'indices=indices_file.txt' or 'angular_range=[a,b]'"
+        )
+    excl_type, excl_val = val.split("=")
+    excl_type = excl_type.strip()
+    excl_val = excl_val.strip()
+    check_supported(excl_type, exclude_projections_type.keys(), "exclude_projections type")
+    if excl_type == "angular_range":
+
+        def _get_range(range_val):
+            for c in ["(", ")", "[", "]"]:
+                range_val = range_val.replace(c, "")
+            r_min, r_max = range_val.split(",")
+            return (float(r_min), float(r_max))
+
+        return {"type": "angular_range", "range": _get_range(excl_val)}
+    else:
+        return {"type": excl_type, "file": excl_val}
+
+
 @validator
 def no_validator(val):
     return val
