@@ -6,12 +6,19 @@ import inspect
 import numpy as np
 import scipy.fft  #  pylint: disable=E0611
 from silx.io import get_data
+from typing import Union, Optional
+import math
+from numbers import Real
+from scipy import ndimage as nd
+
 from ..preproc.flatfield import FlatFieldDataUrls
 from ..estimation.cor import (
     CenterOfRotation,
     CenterOfRotationAdaptiveSearch,
     CenterOfRotationSlidingWindow,
     CenterOfRotationGrowingWindow,
+    CenterOfRotationFourierAngles,
+    CenterOfRotationOctaveAccurate,
 )
 from ..estimation.cor_sino import SinoCorInterface
 from ..estimation.tilt import CameraTilt
@@ -20,8 +27,8 @@ from ..resources.logger import LoggerOrPrint
 from ..resources.utils import extract_parameters
 from ..utils import check_supported, is_int
 from .params import tilt_methods
-from ..resources.dataset_analyzer import get_0_180_radios
-from ..misc.rotation import Rotation
+from ..resources.dataset_analyzer import get_radio_pair
+from ..processing.rotation import Rotation
 from ..io.reader import ChunkReader
 from ..preproc.ccd import Log, CCDFilter
 from ..misc import fourier_filters
@@ -29,23 +36,34 @@ from .params import cor_methods
 from ..io.reader import load_images_from_dataurl_dict
 
 
-def estimate_cor(method, dataset_info, do_flatfield=True, cor_options_str=None, logger=None):
+def estimate_cor(method, dataset_info, do_flatfield=True, cor_options: Optional[Union[str, dict]] = None, logger=None):
     logger = LoggerOrPrint(logger)
-    cor_options_str = cor_options_str or ""
+    cor_options = cor_options or {}
     check_supported(method, list(cor_methods.keys()), "COR estimation method")
     method = cor_methods[method]
 
     # Extract CoR parameters from configuration file
-    try:
-        cor_options = extract_parameters(cor_options_str, sep=";")
-    except Exception as exc:
-        msg = "Could not extract parameters from cor_options: %s" % (str(exc))
-        logger.fatal(msg)
-        raise ValueError(msg)
+    if isinstance(cor_options, str):
+        try:
+            cor_options = extract_parameters(cor_options, sep=";")
+        except Exception as exc:
+            msg = "Could not extract parameters from cor_options: %s" % (str(exc))
+            logger.fatal(msg)
+            raise ValueError(msg)
+    elif isinstance(cor_options, dict):
+        pass
+    else:
+        raise TypeError(f"cor_options_str is expected to be a dict or a str. {type(cor_options)} provided")
 
-    # Dispatch
+    # Dispatch. COR estimation is always expressed in absolute number of pixels (i.e. from the center of the first pixel column)
     if method in CORFinder.search_methods:
-        cor_finder = CORFinder(method, dataset_info, do_flatfield=do_flatfield, cor_options=cor_options, logger=logger)
+        cor_finder = CORFinder(
+            method,
+            dataset_info,
+            do_flatfield=do_flatfield,
+            cor_options=cor_options,
+            logger=logger,
+        )
         estimated_cor = cor_finder.find_cor()
     elif method in SinoCORFinder.search_methods:
         cor_finder = SinoCORFinder(
@@ -102,21 +120,102 @@ class CORFinderBase:
             raise TypeError(
                 f"cor_options is expected to be an optional instance of dict. Get {cor_options} ({type(cor_options)}) instead"
             )
-        self.cor_options = cor_options or {}
+        self.cor_options = {}
+        if isinstance(cor_options, dict):
+            self.cor_options.update(cor_options)
+
+        # tomotools internal meeting 07 feb 2024: Merge of options 'near_pos' and 'side'.
+        # See [minutes](https://gitlab.esrf.fr/tomotools/minutes/-/blob/master/minutes-20240207.md?ref_type=heads)
+
+        detector_width = self.dataset_info.radio_dims[0]
+        default_lookup_side = "right" if self.dataset_info.is_halftomo else "center"
+        near_init = self.cor_options.get("side", None)
+
+        if near_init is None:
+            near_init = default_lookup_side
+
+        if near_init == "from_file":
+            try:
+                near_pos = self.dataset_info.dataset_scanner.estimated_cor_frm_motor  # relative pos in pixels
+                if isinstance(near_pos, Real):
+                    # near_pos += detector_width // 2  # Field in NX is relative.
+                    self.cor_options.update({"near_pos": int(near_pos)})
+                else:
+                    near_init = default_lookup_side
+            except:
+                self.logger.warning(
+                    "COR estimation from motor position absent from NX file. Global search is performed."
+                )
+                near_init = default_lookup_side
+        elif isinstance(near_init, Real):
+            self.cor_options.update({"near_pos": int(near_init)})
+            near_init = "near"  # ???
+        elif near_init == "near":  # Legacy
+            if not isinstance(self.cor_options["near_pos"], Real):
+                self.logger.warning("Side option set to 'near' but no 'near_pos' option set.")
+                self.logger.warning("Set side to right if HA, center otherwise.")
+                near_init = default_lookup_side
+        elif near_init in ("left", "right", "center", "all"):
+            pass
+        else:
+            self.logger.warning(
+                f"COR option 'side' received {near_init} and should be either 'from_file' (default), 'left', 'right', 'center', 'near' or a number."
+            )
+
+        if isinstance(self.cor_options.get("near_pos", None), Real):
+            # Check validity of near_pos
+            if np.abs(self.cor_options["near_pos"]) > detector_width / 2:
+                self.logger.warning(
+                    f"Relative COR passed is greater than half the size of the detector. Did you enter a absolute COR position?"
+                )
+                self.logger.warning("Instead, the center of the detector is used.")
+                self.cor_options["near_pos"] = 0
+
+            # Set side from near_pos if passed.
+            if self.cor_options["near_pos"] < 0.0:
+                self.cor_options.update({"side": "left"})
+                near_init = "left"
+            else:
+                self.cor_options.update({"side": "right"})
+                near_init = "right"
+
+        self.cor_options.update({"side": near_init})
+
+        # At this stage : side is set to one of left, right, center near.
+        # and near_pos to a numeric value.
+
+        # if isinstance(self.cor_options["near_pos"], Real):
+        #    # estimated_cor_frm_motor value is supposed to be relative. Since the config documentation expects the "near_pos" options
+        #    # to be given as an absolute COR estimate, a conversion is needed.
+        #    self.cor_options["near_pos"] += detector_width // 2  # converted in absolute nb of pixels.
+        # if not (isinstance(self.cor_options["near_pos"], Real) or self.cor_options["near_pos"] == "ignore"):
+        #    self.cor_options.update({"near_pos": "ignore"})
+
+        # At this stage, cor_options["near_pos"] is either
+        #   - 'ignore':
+        #   - an (absolute) integer value (either the user-provided one if present or the NX one).
 
         cor_class = self.search_methods[method]["class"]
-        self.cor_finder = cor_class(logger=self.logger)
+        self.cor_finder = cor_class(logger=self.logger, cor_options=self.cor_options)
 
-        default_lookup_side = "right" if self.dataset_info.is_halftomo else "center"
         lookup_side = self.cor_options.get("side", default_lookup_side)
 
+        # OctaveAccurate
+        # if cor_class == CenterOfRotationOctaveAccurate:
+        #    lookup_side = "center"
+        angles = self.dataset_info.rotation_angles
+
         self.cor_exec_args = []
         self.cor_exec_args.extend(self.search_methods[method].get("default_args", []))
 
         # CenterOfRotationSlidingWindow is the only class to have a mandatory argument ("side")
         # TODO - it would be more elegant to have it as a kwarg...
-        if len(self.cor_exec_args) > 0 and cor_class is CenterOfRotationSlidingWindow:
-            self.cor_exec_args[0] = lookup_side
+        if len(self.cor_exec_args) > 0:
+            if cor_class in (CenterOfRotationSlidingWindow, CenterOfRotationOctaveAccurate):
+                self.cor_exec_args[0] = lookup_side
+            elif cor_class in (CenterOfRotationFourierAngles,):
+                self.cor_exec_args[0] = angles
+                self.cor_exec_args[1] = lookup_side
         #
         self.cor_exec_kwargs = update_func_kwargs(self.cor_finder.find_shift, self.cor_options)
 
@@ -141,9 +240,15 @@ class CORFinder(CORFinderBase):
         "growing-window": {
             "class": CenterOfRotationGrowingWindow,
         },
+        "octave-accurate": {
+            "class": CenterOfRotationOctaveAccurate,
+            "default_args": ["center"],
+        },
     }
 
-    def __init__(self, method, dataset_info, do_flatfield=True, cor_options=None, logger=None):
+    def __init__(
+        self, method, dataset_info, do_flatfield=True, cor_options=None, logger=None, radio_angles: tuple = (0.0, np.pi)
+    ):
         """
         Initialize a CORFinder object.
 
@@ -151,15 +256,19 @@ class CORFinder(CORFinderBase):
         ----------
         dataset_info: `nabu.resources.dataset_analyzer.DatasetAnalyzer`
             Dataset information structure
+        radio_angles: angles to use to find the cor
         """
         super().__init__(method, dataset_info, do_flatfield=do_flatfield, cor_options=cor_options, logger=logger)
+        self._radio_angles = radio_angles
         self._init_radios()
         self._init_flatfield()
         self._apply_flatfield()
         self._apply_tilt()
 
     def _init_radios(self):
-        self.radios, self._radios_indices = get_0_180_radios(self.dataset_info, return_indices=True)
+        self.radios, self._radios_indices = get_radio_pair(
+            self.dataset_info, radio_angles=self._radio_angles, return_indices=True
+        )
 
     def _init_flatfield(self):
         if not (self.do_flatfield):
@@ -218,13 +327,14 @@ class SinoCORFinder(CORFinderBase):
         "sino-coarse-to-fine": {
             "class": SinoCorInterface,
         },
-        "sliding-window": {
+        "sino-sliding-window": {
             "class": CenterOfRotationSlidingWindow,
             "default_args": ["right"],
         },
-        "growing-window": {
+        "sino-growing-window": {
             "class": CenterOfRotationGrowingWindow,
         },
+        "fourier-angles": {"class": CenterOfRotationFourierAngles, "default_args": [None, "center"]},
     }
 
     def __init__(
@@ -279,6 +389,7 @@ class SinoCORFinder(CORFinderBase):
                 self.projs_indices = np.round(indices_float).astype(np.int32).tolist()
             else:  # Subsampling step
                 self.projs_indices = projs_idx[::subsampling]
+                self.angles = self.dataset_info.rotation_angles[::subsampling]
         else:  # Angular step
             raise NotImplementedError()
 
@@ -331,7 +442,7 @@ class SinoCORFinder(CORFinderBase):
         self.logger.info("Estimating center of rotation")
         self.logger.debug("%s.find_shift(%s)" % (self.cor_finder.__class__.__name__, str(self.cor_exec_kwargs)))
         img_1, img_2 = self._split_sinogram(self.sinogram)
-        shift = self.cor_finder.find_shift(img_1, img_2, *self.cor_exec_args, **self.cor_exec_kwargs)
+        shift = self.cor_finder.find_shift(img_1, np.fliplr(img_2), *self.cor_exec_args, **self.cor_exec_kwargs)
         return self.shape[1] / 2 + shift
 
 
@@ -339,7 +450,7 @@ class SinoCORFinder(CORFinderBase):
 SinoCOREstimator = SinoCORFinder
 
 
-class CompositeCORFinder:
+class CompositeCORFinder(CORFinderBase):
     """
     Class and method to prepare sinogram and calculate COR
     The pseudo sinogram is built with shrinked radios taken every theta_interval degres
@@ -352,10 +463,15 @@ class CompositeCORFinder:
         - The spike_threshold has been added with default to 0.04
         - The angular sampling is every 5 degree by default, as it is now the case also in the octave script
         - The finding of the optimal overlap is doing by looping over the possible overlap, according to the overlap.
-           After a first testing phase by Elodie we, this part, which is the time consuming part, can be accelerated
+           After a first testing phase, this part, which is the time consuming part, can be accelerated
            by several order of magnitude without modifing the final result
     """
 
+    search_methods = {
+        "composite-coarse-to-fine": {
+            "class": CenterOfRotation,  # Hack. Not used. Everything is done in the find_cor() func.
+        }
+    }
     _default_cor_options = {"low_pass": 0.4, "high_pass": 10, "side": "center", "near_pos": 0, "near_width": 20}
 
     def __init__(
@@ -368,10 +484,28 @@ class CompositeCORFinder:
         cor_options=None,
         spike_threshold=0.04,
         logger=None,
+        norm_order=1,
     ):
+        super().__init__(
+            "composite-coarse-to-fine", dataset_info, do_flatfield=True, cor_options=cor_options, logger=logger
+        )
+        if norm_order not in [1, 2]:
+            raise ValueError(
+                f""" the norm order (nom_order parameter) must be either 1 or 2. You passed {norm_order}
+                """
+            )
+
+        self.norm_order = norm_order
+
         self.dataset_info = dataset_info
         self.logger = LoggerOrPrint(logger)
 
+        self.sx, self.sy = self.dataset_info.radio_dims
+
+        default_cor_options = self._default_cor_options.copy()
+        default_cor_options.update(self.cor_options)
+        self.cor_options = default_cor_options
+
         # the algorithm can work for angular ranges larger than 1.2*pi
         # up to an arbitrarily number of turns as it is the case in helical scans
         self.spike_threshold = spike_threshold
@@ -382,8 +516,6 @@ class CompositeCORFinder:
         self.angle_min = self.unwrapped_rotation_angles.min()
         self.angle_max = self.unwrapped_rotation_angles.max()
 
-        self.sx, self.sy = self.dataset_info.radio_dims
-
         if (self.angle_max - self.angle_min) < 1.2 * np.pi:
             useful_span = None
             raise ValueError(
@@ -398,7 +530,7 @@ class CompositeCORFinder:
             if useful_span < np.pi:
                 theta_interval = theta_interval * useful_span / np.pi
 
-        self._get_cor_options(cor_options)
+        # self._get_cor_options(cor_options)
 
         self.take_log = take_log
         self.ovs = oversampling
@@ -458,7 +590,6 @@ class CompositeCORFinder:
         # initialize sinograms and radios arrays
         self.sino = np.zeros([2 * self.nprobed * n_subsampling_y, (self.sx - 1) * self.ovs + 1], "f")
         self._loaded = False
-
         self.high_pass = self.cor_options["high_pass"]
         img_filter = fourier_filters.get_bandpass_filter(
             (self.sino.shape[0] // 2, self.sino.shape[1]),
@@ -476,16 +607,9 @@ class CompositeCORFinder:
         """oversampling in the horizontal direction"""
         if self.ovs == 1:
             return radio
-        result = np.zeros([radio.shape[0], (radio.shape[1] - 1) * self.ovs + 1], "f")
-        result[:, :: self.ovs] = radio
-
-        for i in range(1, self.ovs):
-            f = i / self.ovs
-            result[:, i :: self.ovs] = (1 - f) * result[:, 0 : -self.ovs : self.ovs] + f * result[
-                :, self.ovs :: self.ovs
-            ]
-
-        return result
+        else:
+            ovs_2D = [1, self.ovs]
+        return oversample(radio, ovs_2D)
 
     def _get_cor_options(self, cor_options):
         default_dict = self._default_cor_options.copy()
@@ -627,46 +751,125 @@ class CompositeCORFinder:
         )
 
         best_overlap = overlap_min
-        best_value = np.inf
+        best_error = np.inf
+
+        blurred_radio1 = nd.gaussian_filter(abs(radio1), [0, self.high_pass])
+        blurred_radio2 = nd.gaussian_filter(abs(radio2), [0, self.high_pass])
+
         for z in range(int(overlap_min), int(overlap_max) + 1):
             if z <= ovsd_sx:
                 my_z = z
                 my_radio1 = radio1
                 my_radio2 = radio2
+                my_blurred_radio1 = blurred_radio1
+                my_blurred_radio2 = blurred_radio2
             else:
                 my_z = ovsd_sx - (z - ovsd_sx)
                 my_radio1 = np.fliplr(radio1)
                 my_radio2 = np.fliplr(radio2)
+                my_blurred_radio1 = np.fliplr(blurred_radio1)
+                my_blurred_radio2 = np.fliplr(blurred_radio2)
 
-            common_left = np.fliplr(my_radio1[:, ovsd_sx - my_z :])[:, : -(self.ovs * self.high_pass * 2)]
-            common_right = my_radio2[:, ovsd_sx - my_z : -(self.ovs * self.high_pass * 2)]
-            common = common_right - common_left
+            common_left = np.fliplr(my_radio1[:, ovsd_sx - my_z :])[:, : -int(math.ceil(self.ovs * self.high_pass * 2))]
+            # adopt a 'safe' margin considering high_pass value (possibly float)
+            common_right = my_radio2[:, ovsd_sx - my_z : -int(math.ceil(self.ovs * self.high_pass * 2))]
 
-            value = np.linalg.norm(common)
-            norm_diff2 = value * value
+            common_blurred_left = np.fliplr(my_blurred_radio1[:, ovsd_sx - my_z :])[
+                :, : -int(math.ceil(self.ovs * self.high_pass * 2))
+            ]
+            # adopt a 'safe' margin considering high_pass value (possibly float)
+            common_blurred_right = my_blurred_radio2[:, ovsd_sx - my_z : -int(math.ceil(self.ovs * self.high_pass * 2))]
 
             if common_right.size == 0:
                 continue
-            norm_right = np.linalg.norm(common_right)
-            norm_left = np.linalg.norm(common_left)
 
-            value = norm_diff2 / (norm_right * norm_left)
+            error = self.error_metric(common_right, common_left, common_blurred_right, common_blurred_left)
 
-            min_value = min(best_value, value)
+            min_error = min(best_error, error)
 
-            if min_value == value:
+            if min_error == error:
                 best_overlap = z
-                best_value = min_value
-            self.logger.debug(
-                "testing an overlap of %.2f pixels, actual best overlap is %.2f pixels over %d\r"
-                % (z / self.ovs, best_overlap / self.ovs, ovsd_sx / self.ovs),
-            )
+                best_error = min_error
+            # self.logger.debug(
+            #     "testing an overlap of %.2f pixels, actual best overlap is %.2f pixels over %d\r"
+            #     % (z / self.ovs, best_overlap / self.ovs, ovsd_sx / self.ovs),
+            # )
 
         offset = (ovsd_sx - best_overlap) / self.ovs / 2
         cor_abs = (self.sx - 1) / 2 + offset
 
         return cor_abs
 
+    def error_metric(self, common_right, common_left, common_blurred_right, common_blurred_left):
+        if self.norm_order == 2:
+            return self.error_metric_l2(common_right, common_left)
+        elif self.norm_order == 1:
+            return self.error_metric_l1(common_right, common_left, common_blurred_right, common_blurred_left)
+        else:
+            assert False, "this cannot happen"
+
+    def error_metric_l2(self, common_right, common_left):
+        common = common_right - common_left
+
+        tmp = np.linalg.norm(common)
+        norm_diff2 = tmp * tmp
+
+        norm_right = np.linalg.norm(common_right)
+        norm_left = np.linalg.norm(common_left)
+
+        res = norm_diff2 / (norm_right * norm_left)
+
+        return res
+
+    def error_metric_l1(self, common_right, common_left, common_blurred_right, common_blurred_left):
+        common = (common_right - common_left) / (common_blurred_right + common_blurred_left)
+
+        res = abs(common).mean()
+
+        return res
+
+
+def oversample(radio, ovs_s):
+    """oversampling an image in arbitrary directions.
+    The first and last point of each axis will still remain as  extremal points of the new axis.
+    """
+    result = np.zeros([(radio.shape[0] - 1) * ovs_s[0] + 1, (radio.shape[1] - 1) * ovs_s[1] + 1], "f")
+
+    # Pre-initialisation: The original data falls exactly on the following strided positions in the new data array.
+    result[:: ovs_s[0], :: ovs_s[1]] = radio
+
+    for k in range(0, ovs_s[0]):
+        # interpolation coefficient for axis 0
+        g = k / ovs_s[0]
+        for i in range(0, ovs_s[1]):
+            if i == 0 and k == 0:
+                # this case subset was already exactly matched from before the present double loop,
+                # in the pre-initialisation line.
+                continue
+            # interpolation coefficent for axis 1
+            f = i / ovs_s[1]
+
+            # stop just a bit before cause we are not extending beyond the limits.
+            # If we are exacly on a vertical or horizontal original line, then no shift will be applied,
+            # and we will exploit the equality f+(1-f)=g+(1-g)=1 adding twice the same contribution with
+            # interpolation factors which become dummies pour le coup.
+            stop0 = -ovs_s[0] if k else None
+            stop1 = -ovs_s[1] if i else None
+
+            # Once again, we exploit the  g+(1-g)=1 equality
+            start0 = ovs_s[0] if k else 0
+            start1 = ovs_s[1] if i else 0
+
+            # and what is done below makes clear the corundum above.
+            result[k :: ovs_s[0], i :: ovs_s[1]] = (1 - g) * (
+                (1 - f) * result[0 : stop0 : ovs_s[0], 0 : stop1 : ovs_s[1]]
+                + f * result[0 : stop0 : ovs_s[0], start1 :: ovs_s[1]]
+            ) + g * (
+                (1 - f) * result[start0 :: ovs_s[0], 0 : stop1 : ovs_s[1]]
+                + f * result[start0 :: ovs_s[0], start1 :: ovs_s[1]]
+            )
+    return result
+
 
 # alias
 CompositeCOREstimator = CompositeCORFinder
@@ -722,7 +925,7 @@ class DetectorTiltEstimator:
             named arguments to pass to the detector tilt estimator class.
         """
         self._set_params(dataset_info, do_flatfield, logger, autotilt_options)
-        self.radios, self.radios_indices = get_0_180_radios(dataset_info, return_indices=True)
+        self.radios, self.radios_indices = get_radio_pair(dataset_info, radio_angles=(0.0, np.pi), return_indices=True)
         self._init_flatfield()
         self._apply_flatfield()
 
