@@ -14,11 +14,12 @@ from ...preproc.shift import VerticalShift
 from ...preproc.double_flatfield import DoubleFlatField
 from ...preproc.phase import PaganinPhaseRetrieval
 from ...preproc.ctf import CTFPhaseRetrieval, GeoPars
-from ...reconstruction.sinogram import SinoBuilder, SinoNormalization
-from ...misc.rotation import Rotation
-from ...reconstruction.rings import MunchDeringer
-from ...misc.unsharp import UnsharpMask
-from ...misc.histogram import PartialHistogram, hist_as_2Darray
+from ...reconstruction.sinogram import SinoNormalization
+from ...reconstruction.filtering import SinoFilter
+from ...processing.rotation import Rotation
+from ...reconstruction.rings import MunchDeringer, SinoMeanDeringer, VoDeringer
+from ...processing.unsharp import UnsharpMask
+from ...processing.histogram import PartialHistogram, hist_as_2Darray
 from ..utils import use_options, pipeline_step, get_subregion
 from ..datadump import DataDumpManager
 from ..writer import WriterManager
@@ -26,7 +27,7 @@ from ..detector_distortion_provider import DetectorDistortionProvider
 
 # For now we don't have a plain python/numpy backend for reconstruction
 try:
-    from ...reconstruction.fbp_opencl import Backprojector
+    from ...reconstruction.fbp_opencl import OpenCLBackprojector as Backprojector
 except:
     Backprojector = None
 
@@ -48,10 +49,12 @@ class ChunkedPipeline:
     UnsharpMaskClass = UnsharpMask
     ImageRotationClass = Rotation
     VerticalShiftClass = VerticalShift
-    SinoDeringerClass = MunchDeringer
+    MunchDeringerClass = MunchDeringer
+    SinoMeanDeringerClass = SinoMeanDeringer
+    VoDeringerClass = VoDeringer
     MLogClass = Log
-    SinoBuilderClass = SinoBuilder
     SinoNormalizationClass = SinoNormalization
+    SinoFilterClass = SinoFilter
     FBPClass = Backprojector
     ConebeamClass = None  # unsupported on CPU
     HistogramClass = PartialHistogram
@@ -249,7 +252,7 @@ class ChunkedPipeline:
 
     @property
     def sino_shape(self):
-        return self.process_config.sino_shape(binning=True, subsampling=True, after_ha=True)
+        return self.process_config.sino_shape(binning=True, subsampling=True)
 
     @property
     def sinos_shape(self):
@@ -424,6 +427,8 @@ class ChunkedPipeline:
             output_is_mlog=False,
             average_is_on_log=avg_is_on_log,
             sigma_filter=options["sigma"],
+            log_clip_min=options["log_min_clip"],
+            log_clip_max=options["log_max_clip"],
         )
 
     @use_options("ccd_correction", "ccd_correction")
@@ -467,7 +472,7 @@ class ChunkedPipeline:
                 lim1=options["ctf_lim1"],
                 lim2=options["ctf_lim2"],
                 logger=self.logger,
-                fftw_num_threads=None,  # TODO tune in advanced params of nabu config file
+                fft_num_threads=None,  # TODO tune in advanced params of nabu config file
                 use_rfft=True,
                 normalize_by_mean=options["ctf_normalize_by_mean"],
                 translation_vh=translations_vh,
@@ -481,12 +486,7 @@ class ChunkedPipeline:
                 pixel_size=options["pixel_size_m"],
                 padding=options["padding_type"],
                 # TODO tune in advanced params of nabu config file
-                fftw_num_threads=None,
-            )
-        if self.phase_retrieval.use_fftw:
-            self.logger.debug(
-                "%s using FFTW with %d threads"
-                % (self.phase_retrieval.__class__.__name__, self.phase_retrieval.fftw.num_threads)
+                fft_num_threads=None,
             )
 
     @use_options("unsharp_mask", "unsharp_mask")
@@ -518,10 +518,25 @@ class ChunkedPipeline:
 
     @use_options("sino_rings_correction", "sino_deringer")
     def _init_sino_rings_correction(self):
+        n_a, n_z, n_x = self.radios_cropped_shape
+        sinos_shape = (n_z, n_a, n_x)
         options = self.processing_options["sino_rings_correction"]
-        fw_params = extract_parameters(options["user_options"])
-        fw_sigma = fw_params.pop("sigma", 1.0)
-        self.sino_deringer = self.SinoDeringerClass(fw_sigma, sinos_shape=self.sinos_shape, **fw_params)
+
+        destriper_params = extract_parameters(options["user_options"])
+        if options["method"] == "munch":
+            # TODO MunchDeringer does not have an API consistent with the other deringers
+            fw_sigma = destriper_params.pop("sigma", 1.0)
+            self.sino_deringer = self.MunchDeringerClass(fw_sigma, sinos_shape, **destriper_params)
+        elif options["method"] == "vo":
+            self.sino_deringer = self.VoDeringerClass(sinos_shape, **destriper_params)
+        elif options["method"] == "mean-subtraction":
+            self.sino_deringer = self.SinoMeanDeringerClass(
+                sinos_shape, mode="subtract", fft_num_threads=None, **destriper_params
+            )
+        elif options["method"] == "mean-division":
+            self.sino_deringer = self.SinoMeanDeringerClass(
+                sinos_shape, mode="divide", fft_num_threads=None, **destriper_params
+            )
 
     @use_options("reconstruction", "reconstruction")
     def _init_reconstruction(self):
@@ -537,20 +552,20 @@ class ChunkedPipeline:
             self.reconstruction = self.FBPClass(
                 self.sinos_shape[1:],
                 angles=options["angles"],
-                rot_center=options["fbp_rotation_axis_position"],
-                filter_name=options["fbp_filter_type"],
+                rot_center=options["rotation_axis_position"],
+                filter_name=options["fbp_filter_type"] or "none",
+                halftomo=options["enable_halftomo"],
                 slice_roi=self.process_config.rec_roi,
                 padding_mode=options["padding_type"],
                 extra_options={
-                    "scale_factor": 1.0 / options["pixel_size_cm"],
+                    "scale_factor": 1.0 / options["voxel_size_cm"][0],
                     "axis_correction": options["axis_correction"],
                     "centered_axis": options["centered_axis"],
                     "clip_outer_circle": options["clip_outer_circle"],
                     "filter_cutoff": options["fbp_filter_cutoff"],
                 },
             )
-            if options["fbp_filter_type"] is None:
-                self.reconstruction.fbp = self.reconstruction.backproj
+
         if options["method"] == "cone":
             radios_shape_for_sino_builder = self.radios_shape
             n_slices = self.n_slices + sum(self.margin[0])
@@ -567,13 +582,10 @@ class ChunkedPipeline:
                 rot_center=options["rotation_axis_position"],
                 pixel_size=1,
             )
-        self.sino_builder = self.SinoBuilderClass(
-            radios_shape=radios_shape_for_sino_builder,
-            rot_center=options["rotation_axis_position"],
-            halftomo=options["enable_halftomo"],
-            angles=options["angles"],
-        )
+
         self._allocate_recs(*self.process_config.rec_shape, n_slices=n_slices)
+        n_a, _, n_x = self.radios_cropped_shape
+        self._tmp_sino = self._allocate_array((n_a, n_x), "f", name="tmp_sino")
         if options["method"] == "cone":
             self.sinos = self._allocate_array(self.sino_builder.output_shape, "f", name="sinos")
 
@@ -583,7 +595,7 @@ class ChunkedPipeline:
         self.histogram = self.HistogramClass(method="fixed_bins_number", num_bins=options["histogram_bins"])
 
     @use_options("save", "writer")
-    def _init_writer(self):
+    def _init_writer(self, **extra_options):
         options = self.processing_options["save"]
         metadata = {
             "process_name": "reconstruction",
@@ -599,7 +611,9 @@ class ChunkedPipeline:
             "float_clip_values": options["float_clip_values"],
             "tiff_single_file": options.get("tiff_single_file", False),
             "single_output_file_initialized": getattr(self.process_config, "single_output_file_initialized", False),
+            "raw_vol_metadata": {"voxelSize": self.dataset_info.pixel_size},  # legacy...
         }
+        writer_extra_options.update(extra_options)
         self.writer = WriterManager(
             options["location"],
             options["file_prefix"],
@@ -707,14 +721,14 @@ class ChunkedPipeline:
         Reconstruction for parallel geometry.
         For each target slice: get the corresponding sinogram, apply some processing, then reconstruct
         """
-        if self.processing_options["reconstruction"]["method"] == "cone":
+        options = self.processing_options["reconstruction"]
+        if options["method"] == "cone":
             self._reconstruct_cone()
             return
+
         for i in range(self.n_slices):
-            sino = self.sino_builder.get_sino(self.radios, i)
-            if self.sino_deringer is not None:
-                self.sino_deringer.remove_rings(sino)
-            self.reconstruction.fbp(sino, output=self.recs[i])
+            self._tmp_sino[:] = self.radios[:, i, :]  # copy into contiguous array
+            self.reconstruction.fbp(self._tmp_sino, output=self.recs[i])
 
     def _reconstruct_cone(self):
         """
@@ -738,8 +752,11 @@ class ChunkedPipeline:
 
     @pipeline_step("writer", "Saving data")
     def _write_data(self, data=None):
-        if data is None:
+        if data is None and self.reconstruction is not None:
             data = self.recs
+        if data is None:
+            self.logger.info("No data to write")
+            return
         self.writer.write_data(data)
         self.logger.info("Wrote %s" % self.writer.fname)
         self._write_histogram()
@@ -794,8 +811,8 @@ class ChunkedPipeline:
         self._radios_movements()
         self._crop_radios()
         self._normalize_sinos()
+        self._destripe_sinos()
         self._dump_sinogram()
-        # self._destripe_sinos()
         self._reconstruct()
         self._compute_histogram()
         self._write_data()
