@@ -5,10 +5,11 @@ from ...preproc.double_flatfield_cuda import CudaDoubleFlatField
 from ...preproc.phase_cuda import CudaPaganinPhaseRetrieval
 from ...preproc.ctf_cuda import CudaCTFPhaseRetrieval
 from ...reconstruction.sinogram_cuda import CudaSinoBuilder, CudaSinoNormalization
-from ...reconstruction.rings_cuda import CudaMunchDeringer
-from ...misc.unsharp_cuda import CudaUnsharpMask
-from ...misc.rotation_cuda import CudaRotation
-from ...misc.histogram_cuda import CudaPartialHistogram
+from ...reconstruction.filtering_cuda import CudaSinoFilter
+from ...reconstruction.rings_cuda import CudaMunchDeringer, CudaSinoMeanDeringer, CudaVoDeringer
+from ...processing.unsharp_cuda import CudaUnsharpMask
+from ...processing.rotation_cuda import CudaRotation
+from ...processing.histogram_cuda import CudaPartialHistogram
 from ...reconstruction.fbp import Backprojector
 from ...reconstruction.cone import __have_astra__, ConebeamReconstructor
 from ...cuda.utils import get_cuda_context, __has_pycuda__, __pycuda_error_msg__
@@ -35,13 +36,16 @@ class CudaChunkedPipeline(ChunkedPipeline):
     UnsharpMaskClass = CudaUnsharpMask
     ImageRotationClass = CudaRotation
     VerticalShiftClass = CudaVerticalShift
-    SinoDeringerClass = CudaMunchDeringer
+    MunchDeringerClass = CudaMunchDeringer
+    SinoMeanDeringerClass = CudaSinoMeanDeringer
+    VoDeringerClass = CudaVoDeringer
     MLogClass = CudaLog
     SinoBuilderClass = CudaSinoBuilder
+    SinoNormalizationClass = CudaSinoNormalization
+    SinoFilterClass = CudaSinoFilter
     FBPClass = Backprojector
     ConebeamClass = ConebeamReconstructor
     HistogramClass = CudaPartialHistogram
-    SinoNormalizationClass = CudaSinoNormalization
 
     def __init__(
         self,
