@@ -3,9 +3,11 @@ from silx.image.tomography import get_next_power
 from ...utils import check_supported
 
 
-def estimate_required_memory(process_config, delta_z=None, delta_a=None, max_mem_allocation_GB=None, debug=False):
+def estimate_required_memory(
+    process_config, delta_z=None, delta_a=None, max_mem_allocation_GB=None, fft_plans=True, debug=False
+):
     """
-    Estimate the memory (RAM) needed for a reconstruction.
+    Estimate the memory (RAM) in Bytes needed for a reconstruction.
 
     Parameters
     -----------
@@ -87,20 +89,31 @@ def estimate_required_memory(process_config, delta_z=None, delta_a=None, max_mem
     # Phase retrieval
     # ---------------
     if "phase" in processing_steps:
-        # Phase retrieval is done image-wise, so near in-place, but needs to
-        # allocate some images, fft plans, and so on
+        # Phase retrieval is done image-wise, so near in-place, but needs to allocate some memory:
+        # filter with padded shape, radio_padded, radio_padded_fourier, and possibly FFT plan.
+        # CTF phase retrieval uses "2 filters" (num and denom) but let's neglect this.
         Nx_p = get_next_power(2 * Nx)
         Nz_p = get_next_power(2 * Nz)
-        img_size_real = 2 * 4 * Nx_p * Nz_p
-        img_size_cplx = 2 * 8 * ((Nx_p * Nz_p) // 2 + 1)
-        total_memory_needed += 2 * img_size_real + 3 * img_size_cplx
+        img_size_real = Nx_p * Nz_p * 4
+        img_size_cplx = ((Nx_p * Nz_p) // 2 + 1) * 8  # assuming RFFT
+        factor = 1
+        if fft_plans:
+            factor = 2
+        total_memory_needed += (2 * img_size_real + img_size_cplx) * factor
 
     # Sinogram de-ringing
     # -------------------
     if "sino_rings_correction" in processing_steps:
-        # Process is done image-wise.
-        # Needs one Discrete Wavelets transform and one FFT/IFFT plan for each scale
-        total_memory_needed += (Nx * Na * 4) * 5.5  # approx.
+        method = process_config.processing_options["sino_rings_correction"]["method"]
+        if method == "munch":
+            # Process is done image-wise.
+            # Needs one Discrete Wavelets transform and one FFT/IFFT plan for each scale
+            factor = 2 if not (fft_plans) else 5.5  # approx!
+            total_memory_needed += (Nx * Na * 4) * factor
+        elif method == "vo":
+            # cupy-based implementation makes many calls to "scipy-like" functions, where the memory usage is not under control
+            # TODO try to estimate this
+            pass
 
     # Reconstruction
     # ---------------
@@ -127,7 +140,14 @@ def estimate_required_memory(process_config, delta_z=None, delta_a=None, max_mem
 
 
 def estimate_max_chunk_size(
-    available_memory_GB, process_config, pipeline_part="all", n_rows=None, step=10, max_mem_allocation_GB=None
+    available_memory_GB,
+    process_config,
+    pipeline_part="all",
+    n_rows=None,
+    step=10,
+    max_mem_allocation_GB=None,
+    fft_plans=True,
+    debug=False,
 ):
     """
     Estimate the maximum size of the data chunk that can be loaded in memory.
@@ -195,7 +215,12 @@ def estimate_max_chunk_size(
     while True:
         try:
             mem = estimate_required_memory(
-                process_config, delta_z=delta_z, delta_a=delta_a, max_mem_allocation_GB=max_mem_allocation_GB
+                process_config,
+                delta_z=delta_z,
+                delta_a=delta_a,
+                max_mem_allocation_GB=max_mem_allocation_GB,
+                fft_plans=fft_plans,
+                debug=debug,
             )
         except ValueError:
             # For very big dataset this function might return "0".
@@ -215,7 +240,17 @@ def estimate_max_chunk_size(
 
     process_config.processing_steps = processing_steps_bak
 
-    res = last_valid_delta_a if pipeline_part == "radios" else last_valid_delta_z
+    if pipeline_part != "radios":
+        if mem / 1e9 < available_memory_GB:
+            res = min(delta_z, process_config.radio_shape()[0])
+        else:
+            res = last_valid_delta_z
+    else:
+        if mem / 1e9 < available_memory_GB:
+            res = min(delta_a, process_config.n_angles())
+        else:
+            res = last_valid_delta_a
+
     # Really not ideal. For very large dataset, "step" should be very small.
     # Otherwise we go from 0 -> OK to 10 -> not OK, and then retain 0...
     if res == 0:
