@@ -9,11 +9,11 @@ from ...io.reader import import_h5_to_dict
 from ...resources.utils import extract_parameters, get_values_from_file
 from ...resources.nxflatfield import update_dataset_info_flats_darks
 from ...resources.utils import get_quantities_and_units
-from ...reconstruction.sinogram import get_extended_sinogram_width
 from ..estimators import estimate_cor
 from ..processconfig import ProcessConfigBase
 from .nabu_config import nabu_config, renamed_keys
 from .dataset_validator import FullFieldDatasetValidator
+from nxtomo.nxobject.nxdetector import ImageKey
 
 
 class ProcessConfig(ProcessConfigBase):
@@ -83,6 +83,10 @@ class ProcessConfig(ProcessConfigBase):
             )
         self.rec_params = self.nabu_config["reconstruction"]
 
+        subsampling_factor, subsampling_start = self.nabu_config["dataset"]["projections_subsampling"]
+        self.subsampling_factor = subsampling_factor or 1
+        self.subsampling_start = subsampling_start or 0
+
         self._update_dataset_with_user_overwrites()
         self._get_rotation_axis_position()
         self._update_rotation_angles()
@@ -118,8 +122,9 @@ class ProcessConfig(ProcessConfigBase):
         if transl_file is not None and "://" not in transl_file:
             try:
                 translations = get_values_from_file(
-                    transl_file, shape=(self.dataset_info.n_angles, last_dim), any_size=True
+                    transl_file, shape=(self.n_angles(subsampling=False), last_dim), any_size=True
                 ).astype(np.float32)
+                translations = translations[self.subsampling_start :: self.subsampling_factor]
             except ValueError:
                 print("Something wrong with translation_movements_file %s" % transl_file)
                 raise
@@ -145,9 +150,10 @@ class ProcessConfig(ProcessConfigBase):
             try:
                 axis_correction = get_values_from_file(
                     axis_correction_file,
-                    n_values=self.dataset_info.n_angles,
+                    n_values=self.n_angles(subsampling=False),
                     any_size=True,
                 ).astype(np.float32)
+                axis_correction = axis_correction[self.subsampling_start :: self.subsampling_factor]
             except ValueError:
                 print("Something wrong with axis correction file %s" % axis_correction_file)
                 raise
@@ -185,7 +191,7 @@ class ProcessConfig(ProcessConfigBase):
                 cor,
                 self.dataset_info,
                 do_flatfield=(self.nabu_config["preproc"]["flatfield"] is not False),
-                cor_options_str=self.nabu_config["reconstruction"]["cor_options"],
+                cor_options=self.nabu_config["reconstruction"]["cor_options"],
                 logger=self.logger,
             )
             self.logger.info("Estimated center of rotation: %.3f" % cor)
@@ -225,7 +231,7 @@ class ProcessConfig(ProcessConfigBase):
         self.logger.debug("Doing coupled validation")
         self._dataset_validator = FullFieldDatasetValidator(self.nabu_config, self.dataset_info)
         # Not so ideal to propagate fields like this
-        for what in ["rec_params", "rec_region", "binning", "subsampling_factor"]:
+        for what in ["rec_params", "rec_region", "binning"]:
             setattr(self, what, getattr(self._dataset_validator, what))
 
     #
@@ -255,7 +261,7 @@ class ProcessConfig(ProcessConfigBase):
     def n_angles(self, subsampling=False):
         rot_angles = self.dataset_info.rotation_angles
         if subsampling:
-            rot_angles = rot_angles[:: (self.subsampling or 1)]
+            rot_angles = rot_angles[self.subsampling_start :: self.subsampling_factor]
         return len(rot_angles)
 
     def radios_shape(self, binning=False, subsampling=False):
@@ -275,7 +281,7 @@ class ProcessConfig(ProcessConfigBase):
             cor = (cor + 0.5) / self.binning_x - 0.5
         return cor
 
-    def sino_shape(self, binning=False, subsampling=False, after_ha=False):
+    def sino_shape(self, binning=False, subsampling=False):
         """
         Return the shape of a sinogram image.
 
@@ -285,21 +291,13 @@ class ProcessConfig(ProcessConfigBase):
             Whether to account for image binning
         subsampling: bool
             Whether to account for projections subsampling
-        after_ha: bool
-            Whether to return the sinogram shape before building the extended-FoV sinogram (default) or after.
         """
         n_a, _, n_x = self.radios_shape(binning=binning, subsampling=subsampling)
-        if self.do_halftomo and after_ha:
-            # int(round(cor/binning)) is not equivalent to int(round(cor))//binning
-            # the former should be used because HA sinograms are built after data is binned
-            cor = self.rotation_axis_position(binning=binning)
-            n_x = get_extended_sinogram_width(n_x, cor)
-            n_a = (n_a + 1) // 2
         return (n_a, n_x)
 
-    def sinos_shape(self, binning=False, subsampling=False, after_ha=False):
+    def sinos_shape(self, binning=False, subsampling=False):
         n_z, _ = self.radio_shape(binning=binning)
-        return (n_z,) + self.sino_shape(binning=binning, subsampling=subsampling, after_ha=after_ha)
+        return (n_z,) + self.sino_shape(binning=binning, subsampling=subsampling)
 
     def projs_indices(self, subsampling=False):
         step = 1
@@ -308,10 +306,12 @@ class ProcessConfig(ProcessConfigBase):
         return sorted(self.dataset_info.projections.keys())[::step]
 
     def rotation_angles(self, subsampling=False):
+        start = 0
         step = 1
         if subsampling:
-            step = self.subsampling or 1
-        return self.dataset_info.rotation_angles[::step]
+            start = self.subsampling_start
+            step = self.subsampling_factor
+        return self.dataset_info.rotation_angles[start::step]
 
     @property
     def rec_roi(self):
@@ -453,6 +453,8 @@ class ProcessConfig(ProcessConfigBase):
             options["double_flatfield"] = {
                 "sigma": nabu_config["preproc"]["dff_sigma"],
                 "processes_file": nabu_config["preproc"]["processes_file"],
+                "log_min_clip": nabu_config["preproc"]["log_min_clip"],
+                "log_max_clip": nabu_config["preproc"]["log_max_clip"],
             }
         #
         # Radios rotation (do it here if possible)
@@ -487,7 +489,11 @@ class ProcessConfig(ProcessConfigBase):
         #
         # Unsharp
         #
-        if nabu_config["phase"]["unsharp_method"] is not None and nabu_config["phase"]["unsharp_coeff"] > 0:
+        if (
+            nabu_config["phase"]["unsharp_method"] is not None
+            and nabu_config["phase"]["unsharp_coeff"] > 0
+            and nabu_config["phase"]["unsharp_sigma"] > 0
+        ):
             tasks.append("unsharp_mask")
             options["unsharp_mask"] = copy_dict_items(
                 nabu_config["phase"], ["unsharp_coeff", "unsharp_sigma", "unsharp_method"]
@@ -514,7 +520,7 @@ class ProcessConfig(ProcessConfigBase):
         translations = dataset_info.translations
         if translations is not None:
             tasks.append("radios_movements")
-            options["radios_movements"] = {"translation_movements": dataset_info.translations[:: self.binning_z]}
+            options["radios_movements"] = {"translation_movements": dataset_info.translations}
         #
         # Sinogram normalization (before half-tomo)
         #
@@ -531,6 +537,7 @@ class ProcessConfig(ProcessConfigBase):
         if nabu_config["preproc"]["sino_rings_correction"]:
             tasks.append("sino_rings_correction")
             options["sino_rings_correction"] = {
+                "method": nabu_config["preproc"]["sino_rings_correction"],
                 "user_options": nabu_config["preproc"]["sino_rings_options"],
             }
         #
@@ -560,24 +567,42 @@ class ProcessConfig(ProcessConfigBase):
             )
             rec_options = options["reconstruction"]
             rec_options["rotation_axis_position"] = self.rotation_axis_position(binning=True)
-            rec_options["fbp_rotation_axis_position"] = rec_options["rotation_axis_position"]
             rec_options["enable_halftomo"] = self.do_halftomo
             rec_options["axis_correction"] = dataset_info.axis_correction
             if dataset_info.axis_correction is not None:
-                rec_options["axis_correction"] = rec_options["axis_correction"][:: self.subsampling_factor]
+                rec_options["axis_correction"] = rec_options["axis_correction"]
+
             rec_options["angles"] = np.array(self.rotation_angles(subsampling=True))
             rec_options["angles"] += np.deg2rad(nabu_config["reconstruction"]["angle_offset"])
-            rec_options["pixel_size_cm"] = dataset_info.pixel_size * 1e-4  # pix size is in microns in dataset_info
+            voxel_size = dataset_info.pixel_size * 1e-4
+            rec_options["voxel_size_cm"] = (
+                voxel_size,
+                voxel_size,
+                voxel_size,
+            )  # pix size is in microns in dataset_info
+
+            # x/y/z position information
+            def get_mean_pos(position_array):
+                if position_array is None:
+                    return None
+                else:
+                    position_array = np.array(position_array)
+                    # filter only projections. Avoid getting noise
+                    position_array = position_array[
+                        np.asarray(dataset_info.dataset_scanner.image_key_control) == ImageKey.PROJECTION.value
+                    ]
+                    return float(np.mean(position_array))
+
+            mean_positions_xyz = (
+                get_mean_pos(dataset_info.dataset_scanner.z_translation),
+                get_mean_pos(dataset_info.dataset_scanner.y_translation),
+                get_mean_pos(dataset_info.dataset_scanner.x_translation),
+            )
+            if all([m is not None for m in mean_positions_xyz]):
+                rec_options["position"] = mean_positions_xyz
             if rec_options["sample_detector_dist"] is None:
                 rec_options["sample_detector_dist"] = self.dataset_info.distance  # was checked to be not None earlier
-            # TODO improve halftomo handling
-            if self.do_halftomo:
-                rec_options["angles"] = rec_options["angles"][: (rec_options["angles"].size + 1) // 2]
-                if rec_options["rotation_axis_position"] < (self.radio_shape(binning=True)[-1] - 1) / 2.0:
-                    rec_options["fbp_rotation_axis_position"] = (
-                        self.radio_shape(binning=True)[-1] - rec_options["rotation_axis_position"]
-                    )
-            # ---
+
             # New key
             rec_options["cor_estimated_auto"] = isinstance(nabu_config["reconstruction"]["rotation_axis_position"], str)
         #
