@@ -5,16 +5,21 @@ import gc
 from psutil import virtual_memory
 from silx.io import get_data
 from silx.io.url import DataUrl
+from tomoscan.esrf.volume.singleframebase import VolumeSingleFrameBase
+from ... import version as nabu_version
 from ...utils import check_supported, subdivide_into_overlapping_segment
 from ...resources.logger import LoggerOrPrint
+from ...resources.utils import is_hdf5_extension
 from ...io.writer import merge_hdf5_files, NXProcessWriter
-from ...cuda.utils import collect_cuda_gpus
+from ...cuda.utils import collect_cuda_gpus, __has_pycuda__
 from ...preproc.phase import compute_paganin_margin
-from ...misc.histogram import PartialHistogram, add_last_bin, hist_as_2Darray
+from ...processing.histogram import PartialHistogram, add_last_bin, hist_as_2Darray
 from .chunked import ChunkedPipeline
-from .chunked_cuda import CudaChunkedPipeline
 from .computations import estimate_max_chunk_size
 
+if __has_pycuda__:
+    from .chunked_cuda import CudaChunkedPipeline
+
 
 def variable_idxlen_sort(fname):
     return int(fname.split("_")[-1].split(".")[0])
@@ -78,6 +83,9 @@ class FullFieldReconstructor:
         self._build_tasks()
 
         self._do_histograms = self.process_config.nabu_config["postproc"]["output_histogram"]
+        self._reconstruction_output_format_is_hdf5 = is_hdf5_extension(
+            self.process_config.nabu_config["output"]["file_format"]
+        )
         self._histogram_merged = False
         self.pipeline = None
         self._histogram_merged = False
@@ -313,7 +321,6 @@ class FullFieldReconstructor:
                 delta_beta=phase_options["delta_beta"],
                 pixel_size=phase_options["pixel_size_m"],
                 padding=phase_options["padding_type"],
-                use_rfft=False,  # disable fftw here
             )
         elif phase_options["method"] == "CTF":
             # The whole projection has to be processed!
@@ -623,6 +630,23 @@ class FullFieldReconstructor:
         local_files = [join(out_cfg["file_prefix"], basename(fname)) for fname in files]
         return local_files
 
+    def _get_reconstruction_metadata(self, partial_volumes_files=None):
+        metadata = {
+            "nabu_config": self.process_config.nabu_config,
+            "processing_options": self.process_config.processing_options,
+        }
+        if self._reconstruction_output_format_is_hdf5 and partial_volumes_files is not None:
+            metadata[self._process_name + "_stages"] = {
+                str(k): v for k, v in zip(self.results.keys(), partial_volumes_files)
+            }
+        if not (self._reconstruction_output_format_is_hdf5):
+            metadata["process_info"] = {
+                "process_name": "reconstruction",
+                "processing_index": 0,
+                "nabu_version": nabu_version,
+            }
+        return metadata
+
     def merge_hdf5_reconstructions(
         self,
         output_file=None,
@@ -699,11 +723,7 @@ class FullFieldReconstructor:
             output_entry=entry,
             output_filemode="a",
             processing_index=0,
-            config={
-                self._process_name + "_stages": {str(k): v for k, v in zip(self.results.keys(), local_files)},
-                "nabu_config": self.process_config.nabu_config,
-                "processing_options": self.process_config.processing_options,
-            },
+            config=self._get_reconstruction_metadata(local_files),
             base_dir=out_dir,
             axis=axis,
             overwrite=out_cfg["overwrite_results"],
@@ -715,8 +735,6 @@ class FullFieldReconstructor:
             user_rec_config["end_z"] = old_end_z
         return output_file
 
-    merge_hdf5_files = merge_hdf5_reconstructions
-
     def merge_histograms(self, output_file=None, force_merge=False):
         """
         Merge the partial histograms
@@ -796,7 +814,7 @@ class FullFieldReconstructor:
         # Merge HDF5 files
         for step_name, files in dumps.items():
             dump_file = self.process_config.get_save_steps_file(step_name=step_name)
-            self.merge_hdf5_files(
+            self.merge_hdf5_reconstructions(
                 output_file=dump_file,
                 output_dir=dirname(dump_file),
                 files=files,
@@ -804,3 +822,30 @@ class FullFieldReconstructor:
                 axis=axis,
                 merge_histograms=False,
             )
+
+    def write_metadata_file(self):
+        metadata = self._get_reconstruction_metadata()
+        save_options = self.process_config.processing_options["save"]
+        # Perhaps there is more elegant
+        metadata_writer = VolumeSingleFrameBase(
+            url=DataUrl(file_path=save_options["location"], data_path="/"),
+            volume_basename=save_options["file_prefix"],
+            overwrite=True,
+            metadata=metadata,
+        )
+        metadata_writer.save_metadata()
+
+    def finalize_files_saving(self):
+        """
+        Last step to save data. This will do several things:
+          - Merge data dumps (which are always HDF5 files): create a master file for all data-dump sub-volumes
+          - Merge HDF5 reconstruction (if output format is HDF5)
+          - Create a "metadata file" (if output format is not HDF5)
+          - Merge histograms (if output format is not HDF5)
+        """
+        self.merge_data_dumps()
+        if self._reconstruction_output_format_is_hdf5:
+            self.merge_hdf5_reconstructions()
+        else:
+            self.merge_histograms()
+            self.write_metadata_file()
