@@ -1,23 +1,24 @@
-from os.path import join, isfile, basename, dirname
+from os.path import join, isfile, dirname
 from math import ceil
-import gc
 from time import time
-from psutil import virtual_memory
-from silx.io import get_data
-from silx.io.url import DataUrl
 import numpy as np
-import math
 import copy
 from ...resources.logger import LoggerOrPrint
-from ...io.writer import merge_hdf5_files, NXProcessWriter
+from ...io.writer import merge_hdf5_files
 from ...cuda.utils import collect_cuda_gpus
-from ...preproc.phase import compute_paganin_margin
-from ...misc.histogram import PartialHistogram, add_last_bin, hist_as_2Darray
-from .span_strategy import SpanStrategy
+
+try:
+    import nabuxx
+
+    SpanStrategy = nabuxx.span_strategy.SpanStrategy
+except:
+    logger_tmp = LoggerOrPrint(None)
+    logger_tmp.info("Nabuxx not available. Loading python implementation for SpanStrategy")
+    from .span_strategy import SpanStrategy
 
 from .helical_chunked_regridded_cuda import CudaHelicalChunkedRegriddedPipeline
 
-from ..fullfield.reconstruction import collect_cuda_gpus, variable_idxlen_sort, FullFieldReconstructor
+from ..fullfield.reconstruction import collect_cuda_gpus, FullFieldReconstructor
 
 avail_gpus = collect_cuda_gpus() or {}
 
@@ -62,12 +63,20 @@ class HelicalReconstructorRegridded:
            - "max_chunk_size": None,
            - "phase_margin": None,
            - "dry_run": 0,
+           - "diag_zpro_run": 0,
         """
         self.logger = LoggerOrPrint(logger)
         self.process_config = process_config
         self._set_extra_options(extra_options)
         self._get_resources()
-        self._margin_v, self._margin_h = self._compute_phase_margin()
+
+        ### intrication problem: this is used in fullfield's compute_margin to clamp the margin but not used by the present pipeline
+        ### Set it to a big number that will never clamp
+        self.n_z = 10000000  # a big number
+        self.n_x = 10000000  # a big number
+        ###
+        self._compute_margin()
+        # self._margin_v, self._margin_h = self._compute_phase_margin()
 
         self._setup_span_info()
         self._compute_max_chunk_size()
@@ -87,6 +96,7 @@ class HelicalReconstructorRegridded:
             "max_chunk_size": None,
             "phase_margin": None,
             "dry_run": 0,
+            "diag_zpro_run": 0,
         }
         advanced_options.update(extra_options)
         self.extra_options = advanced_options
@@ -95,8 +105,15 @@ class HelicalReconstructorRegridded:
         self.use_phase_margin = self.extra_options["use_phase_margin"]
 
         self.dry_run = self.extra_options["dry_run"]
+        self.diag_zpro_run = self.extra_options["diag_zpro_run"]
 
         self._do_histograms = self.process_config.nabu_config["postproc"]["output_histogram"]
+
+        if self.diag_zpro_run:
+            self.process_config.processing_options.get("phase", None)
+            self._do_histograms = False
+            self.reading_granularity = 10
+
         self._histogram_merged = False
         self._span_info = None
 
@@ -104,6 +121,8 @@ class HelicalReconstructorRegridded:
         rec_cfg = self.process_config.nabu_config["reconstruction"]
         self.z_min = rec_cfg["start_z"]
         self.z_max = rec_cfg["end_z"] + 1
+        z_fract_min = rec_cfg["start_z_fract"]
+        z_fract_max = rec_cfg["end_z_fract"]
         z_min_mm = rec_cfg["start_z_mm"]
         z_max_mm = rec_cfg["end_z_mm"]
 
@@ -141,11 +160,25 @@ class HelicalReconstructorRegridded:
                     if (z_min_mm - z_mm_s[i]) * (z_min_mm - z_mm_s[i + 1]) <= 0:
                         self.z_min = len(z_mm_s) - 1 - i
                         break
+        elif z_fract_min != 0.0 or z_fract_max != 0.0:
+            z_start, z_max = (self._span_info.get_doable_span()).view_heights_minmax
+
+            # the meaming of z_min and z_max is: position in slices units from the
+            # first available slice and in the direction of the scan
+            self.z_min = int(round(z_start * (0 - z_fract_min) + z_max * z_fract_min))
+            self.z_max = int(round(z_start * (0 - z_fract_max) + z_max * z_fract_max)) + 1
+
+    def _compute_translations_margin(self):
+        return 0, 0
+
+    def _compute_cone_overlap(self):
+        return 0, 0
 
     _get_resources = FullFieldReconstructor._get_resources
     _get_memory = FullFieldReconstructor._get_memory
     _get_gpu = FullFieldReconstructor._get_gpu
     _compute_phase_margin = FullFieldReconstructor._compute_phase_margin
+    _compute_margin = FullFieldReconstructor._compute_margin
     _compute_unsharp_margin = FullFieldReconstructor._compute_unsharp_margin
     _print_tasks = FullFieldReconstructor._print_tasks
     _instantiate_pipeline_if_necessary = FullFieldReconstructor._instantiate_pipeline_if_necessary
@@ -174,6 +207,14 @@ class HelicalReconstructorRegridded:
 
         user_max_chunk_size = self.extra_options["max_chunk_size"]
 
+        if self.diag_zpro_run:
+            if user_max_chunk_size is not None:
+                user_max_chunk_size = min(
+                    user_max_chunk_size, max(self.process_config.dataset_info.radio_dims[1] // 4, 10)
+                )
+            else:
+                user_max_chunk_size = max(self.process_config.dataset_info.radio_dims[1] // 4, 10)
+
         self.cpu_max_chunk_size = self.estimate_chunk_size(
             cpu_mem, self.process_config, chunk_step=1, user_max_chunk_size=user_max_chunk_size
         )
@@ -213,12 +254,14 @@ class HelicalReconstructorRegridded:
                 reading_granularity=self.reading_granularity,
                 margin_v=self._margin_v,
                 span_info=self._span_info,
+                diag_zpro_run=self.diag_zpro_run,
             )
 
             required_mem_GB = required_mem / 1e9
             if required_mem_GB > available_memory_GB:
                 break
             last_good_chunk_size = chunk_size
+
             if user_max_chunk_size is not None and chunk_size > user_max_chunk_size:
                 last_good_chunk_size = user_max_chunk_size
                 break
@@ -264,7 +307,7 @@ class HelicalReconstructorRegridded:
                 """
                 raise ValueError(message)
 
-            if self._span_info.z_pix_per_proj[-1] > self._span_info.z_pix_per_proj[0]:
+            if self._span_info.z_pix_per_proj[-1] >= self._span_info.z_pix_per_proj[0]:
                 my_z_min = z_start + self.z_min
                 my_z_end = z_start + self.z_max
             else:
@@ -277,7 +320,8 @@ class HelicalReconstructorRegridded:
         print("my_z_min my_z_end ", my_z_min, my_z_end)
         if my_z_min >= my_z_end:
             message = f""" The requested vertical span, after translation to absolute doable heights would be {my_z_min, my_z_end}
-            is not  doable (start>=end)
+            is not  doable (start>=end).
+            Scans are often shorter than expected ThereFore :  CONSIDER TO INCREASE angular_tolerance_steps
             """
             raise ValueError(message)
 
@@ -400,7 +444,6 @@ class HelicalReconstructorRegridded:
             phase_margin_pix=self._margin_v,
             projection_angles_deg=angles_deg,
             pixel_size_mm=self.process_config.dataset_info.pixel_size * 1.0e-3,  # micron to mm
-            logger=self.logger,
             require_redundancy=(redundancy_angle_deg > 0),
             angular_tolerance_steps=self.process_config.nabu_config["reconstruction"]["angular_tolerance_steps"],
         )
@@ -424,7 +467,8 @@ class HelicalReconstructorRegridded:
             logger=self.logger,
             phase_margin=task["phase_margin"],
             reading_granularity=self.reading_granularity,
-            span_info=self._span_info
+            span_info=self._span_info,
+            diag_zpro_run=self.diag_zpro_run
             # cuda_options=self.cuda_options
         )
 
