@@ -1,9 +1,9 @@
 import os
 from .config import parse_nabu_config_file
-from ..utils import deprecation_warning, is_writeable
+from ..utils import is_writeable
 from ..resources.logger import Logger, PrinterLogger
 from .config import validate_config
-from ..resources.dataset_analyzer import analyze_dataset, _tomoscan_has_nxversion
+from ..resources.dataset_analyzer import analyze_dataset
 from .estimators import DetectorTiltEstimator
 
 
@@ -21,8 +21,6 @@ class ProcessConfigBase:
         conf_fname=None,
         conf_dict=None,
         dataset_info=None,
-        checks=True,
-        remove_unused_radios=True,
         create_logger=False,
     ):
         """
@@ -139,11 +137,8 @@ class ProcessConfigBase:
             extra_options = {
                 "exclude_projections": self.nabu_config["dataset"]["exclude_projections"],
                 "hdf5_entry": self.nabu_config["dataset"]["hdf5_entry"],
+                "nx_version": self.nabu_config["dataset"]["nexus_version"],
             }
-            if _tomoscan_has_nxversion:  # legacy - should become "if True" soon
-                extra_options["nx_version"] = self.nabu_config["dataset"]["nexus_version"]
-            else:
-                self.logger.warning("Cannot use 'nx_version' for browsing dataset: need tomoscan > 0.6.0")
             self.dataset_info = analyze_dataset(
                 self.nabu_config["dataset"]["location"], extra_options=extra_options, logger=self.logger
             )
