@@ -1,9 +1,11 @@
-from typing import Union
 import numpy as np
 from ..preproc.ccd import CCDFilter, Log
-from ..cuda.kernel import CudaKernel
-from ..cuda.medfilt import MedianFilter
+from ..processing.medfilt_cuda import MedianFilter
 from ..utils import get_cuda_srcfile, updiv, deprecated_class
+from ..cuda.utils import __has_pycuda__
+
+if __has_pycuda__:
+    from ..cuda.kernel import CudaKernel
 
 # COMPAT.
 from .flatfield_cuda import (
@@ -27,11 +29,11 @@ FlatFieldDataUrls = deprecated_class(
 class CudaCCDFilter(CCDFilter):
     def __init__(
         self,
-        radios_shape: tuple,
-        correction_type: str = "median_clip",
-        median_clip_thresh: float = 0.1,
+        radios_shape,
+        correction_type="median_clip",
+        median_clip_thresh=0.1,
         abs_diff=False,
-        cuda_options: Union[dict, None] = None,
+        cuda_options=None,
     ):
         """
         Initialize a CudaCCDCorrection instance.
