@@ -1,9 +1,10 @@
 import math
 import numpy as np
+from scipy.fft import rfft2, irfft2, fft2, ifft2
 from ..resources.logger import LoggerOrPrint
 from ..misc import fourier_filters
 from ..misc.padding import pad_interpolate, recut
-from ..utils import get_num_threads
+from ..utils import get_num_threads, deprecation_warning
 
 
 class GeoPars:
@@ -111,6 +112,7 @@ class CTFPhaseRetrieval:
         lim2=0.2,
         use_rfft=False,
         fftw_num_threads=None,
+        fft_num_threads=None,
         logger=None,
     ):
         """
@@ -138,10 +140,11 @@ class CTFPhaseRetrieval:
         use_rfft: bool, optional
             Whether to use real-to-complex (R2C) FFT instead of usual complex-to-complex (C2C).
         fftw_num_threads: bool or None or int, optional
-            If False is passed: don't use FFTW.
-            If None is passed: use all available threads.
-            If a number is provided: number of threads to use for FFTW.
-            You can pass a negative number to use N - fftw_num_threads cores.
+            DEPRECATED - please use fft_num_threads instead.
+        fft_num_threads: bool or None or int, optional
+            Number of threads to use for FFT.
+            If a number is provided: number of threads to use for FFT.
+            You can pass a negative number to use N - fft_num_threads cores.
         logger: optional
             a logger object
         """
@@ -152,12 +155,18 @@ class CTFPhaseRetrieval:
         self._calc_shape(shape, padded_shape, padding_mode)
         self.delta_beta = delta_beta
 
+        # COMPAT.
+        if fftw_num_threads is not None:
+            deprecation_warning("'fftw_num_threads' is replaced with 'fft_num_threads'", func_name="ctf_fftw")
+            fft_num_threads = fftw_num_threads
+        # ---
+
         self.lim = None
         self.lim1 = lim1
         self.lim2 = lim2
         self.normalize_by_mean = normalize_by_mean
         self.translation_vh = translation_vh
-        self._setup_fft(use_rfft, fftw_num_threads)
+        self._setup_fft(use_rfft, fft_num_threads)
         self._get_ctf_filter()
 
     def _calc_shape(self, shape, padded_shape, padding_mode):
@@ -175,25 +184,11 @@ class CTFPhaseRetrieval:
         self.shape_padded = tuple(padded_shape)
         self.padding_mode = padding_mode
 
-    def _setup_fft(self, use_rfft, fftw_num_threads):
+    def _setup_fft(self, use_rfft, fft_num_threads):
         self.use_rfft = use_rfft
-        self._fft_func = np.fft.rfft2 if use_rfft else np.fft.fft2
-        self._ifft_func = np.fft.irfft2 if use_rfft else np.fft.ifft2
-        self.use_fftw = False
-        if fftw_num_threads is False:
-            return
-        fftw_num_threads = get_num_threads(fftw_num_threads)
-        if self.use_rfft and (fftw_num_threads > 0):
-            # importing silx.math.fft creates opencl contexts all over the place
-            # because of the silx.opencl.ocl singleton.
-            # So, import silx as late as possible
-            from silx.math.fft.fftw import FFTW, __have_fftw__
-
-            if __have_fftw__:
-                self.use_fftw = True
-                self.fftw = FFTW(shape=self.shape_padded, dtype="f", num_threads=fftw_num_threads)
-                self._fft_func = self.fftw.fft
-                self._ifft_func = self.fftw.ifft
+        self._fft_func = rfft2 if use_rfft else fft2
+        self._ifft_func = irfft2 if use_rfft else ifft2
+        self.fft_num_threads = get_num_threads(fft_num_threads)
 
     def _get_ctf_filter(self):
         """
@@ -320,7 +315,7 @@ class CTFPhaseRetrieval:
         self._ctf_filter_denom = (2 * self.unreg_filter_denom * self.unreg_filter_denom + self.lim).astype(np.complex64)
 
     def _apply_filter(self, img):
-        img_f = self._fft_func(img)
+        img_f = self._fft_func(img, workers=self.fft_num_threads)
         img_f *= self.unreg_filter_denom
 
         unreg_filter_denom_0_mean = self.unreg_filter_denom[0, 0]
@@ -331,7 +326,7 @@ class CTFPhaseRetrieval:
 
         ## formula 8, with regularisation to stay at a safe distance from the poles
         img_f /= self._ctf_filter_denom
-        ph = self._ifft_func(img_f).real
+        ph = self._ifft_func(img_f, workers=self.fft_num_threads).real
         return ph
 
     def retrieve_phase(self, img, output=None):
