@@ -5,6 +5,7 @@ from silx.io.url import DataUrl
 from ..utils import check_supported, check_shape, get_2D_3D_shape
 from ..io.reader import Readers
 from ..io.writer import Writers
+from .ccd import Log
 
 
 class DoubleFlatField:
@@ -22,6 +23,8 @@ class DoubleFlatField:
         average_is_on_log=False,
         sigma_filter=None,
         filter_mode="reflect",
+        log_clip_min=None,
+        log_clip_max=None,
     ):
         """
         Init double flat field by summing a series of urls and considering the same subregion of them.
@@ -55,6 +58,8 @@ class DoubleFlatField:
         self.radios_shape = get_2D_3D_shape(shape)
         self.n_angles = self.radios_shape[0]
         self.shape = self.radios_shape[1:]
+        self._log_clip_min = log_clip_min
+        self._log_clip_max = log_clip_max
         self._init_filedump(result_url, sub_region, detector_corrector)
         self._init_processing(input_is_mlog, output_is_mlog, average_is_on_log, sigma_filter, filter_mode)
         self._computed = False
@@ -112,17 +117,19 @@ class DoubleFlatField:
             self.sigma_filter = None
         self.filter_mode = filter_mode
         proc = lambda x, o: np.copyto(o, x)
+        self._mlog = Log((1,) + self.shape, clip_min=self._log_clip_min, clip_max=self._log_clip_max)
+
         if self.input_is_mlog:
             if not self.average_is_on_log:
                 proc = lambda x, o: np.exp(-x, out=o)
         else:
             if self.average_is_on_log:
-                proc = lambda x, o: -np.log(x, out=o)
+                proc = self._proc_mlog
 
         postproc = lambda x: x
         if self.output_is_mlog:
             if not self.average_is_on_log:
-                postproc = lambda x: -np.log(x)
+                postproc = self._proc_mlog
         else:
             if self.average_is_on_log:
                 postproc = lambda x: np.exp(-x)
@@ -130,6 +137,11 @@ class DoubleFlatField:
         self.proc = proc
         self.postproc = postproc
 
+    def _proc_mlog(self, x, o):
+        o[:] = x[:]
+        self._mlog.take_logarithm(o)
+        return o
+
     def compute_double_flatfield(self, radios, recompute=False):
         """
         Read the radios and generate the "double flat field" by averaging
