@@ -2,7 +2,8 @@ from .double_flatfield import DoubleFlatField
 from ..utils import check_shape
 from ..cuda.utils import __has_pycuda__
 from ..cuda.processing import CudaProcessing
-from ..misc.unsharp_cuda import CudaUnsharpMask
+from ..processing.unsharp_cuda import CudaUnsharpMask
+from .ccd_cuda import CudaLog
 
 if __has_pycuda__:
     import pycuda.gpuarray as garray
@@ -21,6 +22,8 @@ class CudaDoubleFlatField(DoubleFlatField):
         average_is_on_log=False,
         sigma_filter=None,
         filter_mode="reflect",
+        log_clip_min=None,
+        log_clip_max=None,
         cuda_options=None,
     ):
         """
@@ -37,6 +40,8 @@ class CudaDoubleFlatField(DoubleFlatField):
             average_is_on_log=average_is_on_log,
             sigma_filter=sigma_filter,
             filter_mode=filter_mode,
+            log_clip_min=log_clip_min,
+            log_clip_max=log_clip_max,
         )
         self._init_gaussian_filter()
 
@@ -57,16 +62,6 @@ class CudaDoubleFlatField(DoubleFlatField):
         cumath.exp(o, out=o)
         return o
 
-    @staticmethod
-    def _proc_mlog(x, o, min_clip=None):
-        if min_clip is not None:
-            garray.maximum(x, min_clip, out=o)
-            cumath.log(o, out=o)
-        else:
-            cumath.log(x, out=o)
-        o *= -1
-        return o
-
     def _init_processing(self, input_is_mlog, output_is_mlog, average_is_on_log, sigma_filter, filter_mode):
         self.input_is_mlog = input_is_mlog
         self.output_is_mlog = output_is_mlog
@@ -77,6 +72,7 @@ class CudaDoubleFlatField(DoubleFlatField):
         self.filter_mode = filter_mode
         # proc = lambda x,o: np.copyto(o, x)
         proc = self._proc_copy
+        self._mlog = CudaLog((1,) + self.shape, clip_min=self._log_clip_min, clip_max=self._log_clip_max)
         if self.input_is_mlog:
             if not self.average_is_on_log:
                 # proc = lambda x,o: np.exp(-x, out=o)
