@@ -1,6 +1,7 @@
 from math import pi
 from bisect import bisect
 import numpy as np
+from scipy.fft import rfft2, irfft2, fft2, ifft2
 from ..utils import generate_powers, get_decay, check_supported, get_num_threads, deprecation_warning
 
 #
@@ -53,6 +54,7 @@ class PaganinPhaseRetrieval:
         use_rfft=True,
         use_R2C=None,
         fftw_num_threads=None,
+        fft_num_threads=None,
     ):
         """
         Paganin Phase Retrieval for an infinitely distant point source.
@@ -113,9 +115,11 @@ class PaganinPhaseRetrieval:
         use_R2C: bool, optional
             DEPRECATED, use use_rfft instead
         fftw_num_threads: bool or None or int, optional
-            Whether to use FFTW for speeding up FFT.
+            DEPRECATED - please use fft_num_threads
+        fft_num_threads: bool or None or int, optional
+            Number of threads for FFT.
             Default is to use all available threads. You can pass a negative number
-            to use N - fftw_num_threads cores.
+            to use N - fft_num_threads cores.
 
         Important
         ----------
@@ -171,8 +175,11 @@ class PaganinPhaseRetrieval:
         # COMPAT.
         if use_R2C is not None:
             deprecation_warning("'use_R2C' is replaced with 'use_rfft'", func_name="pag_r2c")
-        # -
-        self._get_fft(use_rfft, fftw_num_threads)
+        if fftw_num_threads is not None:
+            deprecation_warning("'fftw_num_threads' is replaced with 'fft_num_threads'", func_name="pag_fftw")
+            fft_num_threads = fftw_num_threads
+        # ---
+        self._get_fft(use_rfft, fft_num_threads)
         self.compute_filter()
 
     def _init_parameters(self, distance, energy, pixel_size, delta_beta, padding):
@@ -191,28 +198,16 @@ class PaganinPhaseRetrieval:
             "reflect": self._pad_reflect,
         }
 
-    def _get_fft(self, use_rfft, fftw_num_threads):
+    def _get_fft(self, use_rfft, fft_num_threads):
         self.use_rfft = use_rfft
         self.use_R2C = use_rfft  # Compat.
-        fftw_num_threads = get_num_threads(fftw_num_threads)
+        self.fft_num_threads = get_num_threads(fft_num_threads)
         if self.use_rfft:
-            self.fft_func = np.fft.rfft2
-            self.ifft_func = np.fft.irfft2
+            self.fft_func = rfft2
+            self.ifft_func = irfft2
         else:
-            self.fft_func = np.fft.fft2
-            self.ifft_func = np.fft.ifft2
-        self.use_fftw = False
-        if self.use_rfft and (fftw_num_threads > 0):
-            # importing silx.math.fft creates opencl contexts all over the place
-            # because of the silx.opencl.ocl singleton.
-            # So, import silx as late as possible
-            from silx.math.fft.fftw import FFTW, __have_fftw__
-
-            if __have_fftw__:
-                self.use_fftw = True
-                self.fftw = FFTW(shape=self.shape_padded, dtype="f", num_threads=fftw_num_threads)
-                self.fft_func = self.fftw.fft
-                self.ifft_func = self.fftw.ifft
+            self.fft_func = fft2
+            self.ifft_func = ifft2
 
     def _calc_shape(self, shape, margin):
         if np.isscalar(shape):
@@ -378,9 +373,9 @@ class PaganinPhaseRetrieval:
 
     def apply_filter(self, radio, padding_method=None, output=None):
         self.pad_data(radio, padding_method=padding_method)
-        radio_f = self.fft_func(self.data_padded)
+        radio_f = self.fft_func(self.data_padded, workers=self.fft_num_threads)
         radio_f *= self.paganin_filter
-        radio_filtered = self.ifft_func(radio_f).real
+        radio_filtered = self.ifft_func(radio_f, workers=self.fft_num_threads).real
         s0, s1 = self.shape_inner
         ((U, _), (L, _)) = self.margin
         if output is None:
