@@ -1,16 +1,15 @@
 import numpy as np
-from math import sqrt, pi
-from ..utils import updiv, get_cuda_srcfile, _sizeof, check_supported
-from .phase import PaganinPhaseRetrieval
 import pycuda.driver as cuda
-from pycuda import gpuarray as garray
+from ..utils import get_cuda_srcfile, check_supported, docstring
 from ..cuda.processing import CudaProcessing
-from ..cuda.kernel import CudaKernel
+from ..processing.fft_cuda import get_fft_class
+from .phase import PaganinPhaseRetrieval
 
 
 class CudaPaganinPhaseRetrieval(PaganinPhaseRetrieval):
     supported_paddings = ["zeros", "constant", "edge"]
 
+    @docstring(PaganinPhaseRetrieval)
     def __init__(
         self,
         shape,
@@ -21,7 +20,9 @@ class CudaPaganinPhaseRetrieval(PaganinPhaseRetrieval):
         padding="edge",
         margin=None,
         cuda_options=None,
-        fftw_num_threads=None,
+        fftw_num_threads=None,  # COMPAT.
+        fft_num_threads=None,
+        fft_backend="skcuda",
     ):
         """
         Please refer to the documentation of
@@ -38,10 +39,10 @@ class CudaPaganinPhaseRetrieval(PaganinPhaseRetrieval):
             padding=padding,
             margin=margin,
             use_rfft=True,
-            fftw_num_threads=None,
+            fft_num_threads=False,
         )
         self._init_gpu_arrays()
-        self._init_fft()
+        self._init_fft(fft_backend)
         self._init_padding_kernel()
         self._init_mult_kernel()
 
@@ -52,25 +53,23 @@ class CudaPaganinPhaseRetrieval(PaganinPhaseRetrieval):
         return padding
 
     def _init_gpu_arrays(self):
-        self.d_paganin_filter = garray.to_gpu(np.ascontiguousarray(self.paganin_filter, dtype=np.float32))
+        self.d_paganin_filter = self.cuda_processing.to_device(
+            "d_paganin_filter", np.ascontiguousarray(self.paganin_filter, dtype=np.float32)
+        )
 
     # overwrite parent method, don't initialize any FFT plan
-    def _get_fft(self, use_rfft, fftw_num_threads):
+    def _get_fft(self, use_rfft, fft_num_threads):
         self.use_rfft = use_rfft
-        self.use_fftw = False
 
-    def _init_fft(self):
-        # Import has to be done here, otherwise scikit-cuda creates a cuda/cublas context at import
-        from silx.math.fft.cufft import CUFFT
-
-        #
-        self.cufft = CUFFT(template=self.data_padded.astype("f"))
-        self.d_radio_padded = self.cufft.data_in
-        self.d_radio_f = self.cufft.data_out
+    def _init_fft(self, fft_backend):
+        fft_cls = get_fft_class(backend=fft_backend)
+        self.cufft = fft_cls(shape=self.data_padded.shape, dtype=np.float32, r2c=True)
+        self.d_radio_padded = self.cuda_processing.allocate_array("d_radio_padded", self.cufft.shape, "f")
+        self.d_radio_f = self.cuda_processing.allocate_array("d_radio_f", self.cufft.shape_out, np.complex64)
 
     def _init_padding_kernel(self):
         kern_signature = {"constant": "Piiiiiiiiffff", "edge": "Piiiiiiii"}
-        self.padding_kernel = CudaKernel(
+        self.padding_kernel = self.cuda_processing.kernel(
             "padding_%s" % self.padding,
             filename=get_cuda_srcfile("padding.cu"),
             signature=kern_signature[self.padding],
@@ -93,7 +92,7 @@ class CudaPaganinPhaseRetrieval(PaganinPhaseRetrieval):
             self.padding_kernel_args.extend([0, 0, 0, 0])
 
     def _init_mult_kernel(self):
-        self.cpxmult_kernel = CudaKernel(
+        self.cpxmult_kernel = self.cuda_processing.kernel(
             "inplace_complexreal_mul_2Dby2D",
             filename=get_cuda_srcfile("ElementOp.cu"),
             signature="PPii",
@@ -110,7 +109,7 @@ class CudaPaganinPhaseRetrieval(PaganinPhaseRetrieval):
         assert data.dtype == np.float32
         # Rectangular memcopy
         # TODO profile, and if needed include this copy in the padding kernel
-        if isinstance(data, np.ndarray) or isinstance(data, garray.GPUArray):
+        if isinstance(data, np.ndarray) or isinstance(data, self.cuda_processing.array_class):
             self.d_radio_padded[: self.shape[0], : self.shape[1]] = data[:, :]
         elif isinstance(data, cuda.DeviceAllocation):
             # TODO manual memcpy2D
