@@ -1,7 +1,6 @@
 import pytest
 import numpy as np
 import scipy.interpolate
-from scipy.misc import ascent
 from nabu.testutils import get_data as nabu_get_data
 from nabu.testutils import __do_long_tests__
 from nabu.preproc.flatfield import FlatFieldArrays
@@ -214,15 +213,15 @@ class TestCtf:
         phase_r2c = ctf_numpy.retrieve_phase(img)
         self.check_result(phase_r2c, self.ref_plain, "Something wrong with CtfFilter-R2C")
 
-        # Test FFTW
-        ctf_fftw = ctf.CtfFilter(*ctf_args, **ctf_kwargs, use_rfft=True, fftw_num_threads=-1)
-        if ctf_fftw.use_rfft:
-            phase_fftw = ctf_fftw.retrieve_phase(img)
-            self.check_result(phase_r2c, self.ref_plain, "Something wrong with CtfFilter-FFTW")
+        # Test multi-core FFT
+        ctf_fft = ctf.CtfFilter(*ctf_args, **ctf_kwargs, use_rfft=True, fft_num_threads=0)
+        if ctf_fft.use_rfft:
+            phase_fft = ctf_fft.retrieve_phase(img)
+            self.check_result(phase_r2c, self.ref_plain, "Something wrong with CtfFilter-FFT")
 
     @pytest.mark.skipif(not (__has_pycuda__ and __has_cufft__), reason="pycuda and scikit-cuda")
     def test_cuda_ctf(self):
-        data = ascent().astype("f")
+        data = nabu_get_data("brain_phantom.npz")["data"]
         delta_beta = 50.0
         energy_kev = 22.0
         distance_m = 1.0
