@@ -1,14 +1,43 @@
 from math import pi
-import pycuda.gpuarray as garray
 import numpy as np
+from scipy.fft import rfft, irfft
 from silx.image.tomography import compute_fourier_filter, get_next_power
-from ..cuda.kernel import CudaKernel
-from ..cuda.processing import CudaProcessing
-from ..utils import get_cuda_srcfile, check_supported, updiv
+from ..processing.padding_base import PaddingBase
+from ..utils import check_supported, get_num_threads
+
+# # COMPAT.
+# from .filtering_cuda import CudaSinoFilter
+
+# SinoFilter = deprecated_class(
+#     "From version 2023.1, 'filtering_cuda.CudaSinoFilter' should be used instead of 'filtering.SinoFilter'. In the future, 'filtering.SinoFilter' will be a numpy-only class.",
+#     do_print=True,
+# )(CudaSinoFilter)
+# #
 
 
 class SinoFilter:
-    available_padding_modes = ["zeros", "edges"]
+    available_filters = [
+        "ramlak",
+        "shepp-logan",
+        "cosine",
+        "hamming",
+        "hann",
+        "tukey",
+        "lanczos",
+        "hilbert",
+    ]
+
+    """
+    A class for sinogram filtering.
+    It does the following:
+      - pad input array
+      - Fourier transform each row
+      - multiply with a 1D or 2D filter
+      - inverse Fourier transform
+    """
+
+    available_padding_modes = PaddingBase.supported_modes
+    default_extra_options = {"cutoff": 1.0, "fft_threads": 0}  # use all threads by default
 
     def __init__(
         self,
@@ -16,31 +45,24 @@ class SinoFilter:
         filter_name=None,
         padding_mode="zeros",
         extra_options=None,
-        cuda_options=None,
     ):
-        """
-        Build a sinogram filter process.
-        """
-        self.cuda = CudaProcessing(**(cuda_options or {}))
         self._init_extra_options(extra_options)
+        self._set_padding_mode(padding_mode)
         self._calculate_shapes(sino_shape)
         self._init_fft()
-        self._allocate_memory(sino_shape)
+        self._allocate_memory()
         self._compute_filter(filter_name)
-        self._set_padding_mode(padding_mode)
-        self._init_kernels()
 
     def _init_extra_options(self, extra_options):
-        self.extra_options = {
-            "cutoff": 1.0,
-        }
-        if extra_options is not None:
-            self.extra_options.update(extra_options)
+        self.extra_options = self.default_extra_options.copy()
+        self.extra_options.update(extra_options or {})
 
     def _set_padding_mode(self, padding_mode):
         # Compat.
-        if padding_mode == "edge":
-            padding_mode = "edges"
+        if padding_mode == "edges":
+            padding_mode = "edge"
+        if padding_mode == "zeros":
+            padding_mode = "constant"
         #
         check_supported(padding_mode, self.available_padding_modes, "padding mode")
         self.padding_mode = padding_mode
@@ -57,8 +79,7 @@ class SinoFilter:
         self.sino_shape = sino_shape
         self.n_angles = n_angles
         self.dwidth = dwidth
-        # int() is crucial here ! Otherwise some pycuda arguments (ex. memcpy2D)
-        # will not work with numpy.int64 (as for 2018.X)
+        # Make sure to use int() here, otherwise pycuda/pyopencl will crash in some cases
         self.dwidth_padded = int(get_next_power(2 * self.dwidth))
         self.sino_padded_shape = (n_angles, self.dwidth_padded)
         if self.ndim == 3:
@@ -66,34 +87,27 @@ class SinoFilter:
         sino_f_shape = list(self.sino_padded_shape)
         sino_f_shape[-1] = sino_f_shape[-1] // 2 + 1
         self.sino_f_shape = tuple(sino_f_shape)
-        #
         self.pad_left = (self.dwidth_padded - self.dwidth) // 2
         self.pad_right = self.dwidth_padded - self.dwidth - self.pad_left
 
     def _init_fft(self):
-        # Import has to be done here, otherwise scikit-cuda creates a cuda/cublas context at import
-        from silx.math.fft.cufft import CUFFT
+        pass
 
-        #
-        self.fft = CUFFT(
-            self.sino_padded_shape,
-            dtype=np.float32,
-            axes=(-1,),
-        )
-
-    def _allocate_memory(self, sino_shape):
-        self.d_filter_f = garray.zeros((self.sino_f_shape[-1],), np.complex64)
-        self.d_sino_padded = self.fft.data_in
-        self.d_sino_f = self.fft.data_out
+    def _allocate_memory(self):
+        pass
 
     def set_filter(self, h_filt, normalize=True):
         """
         Set a filter for sinogram filtering.
 
-        :param h_filt: Filter. Each line of the sinogram will be filtered with
+        Parameters
+        ----------
+        h_filt: numpy.ndarray
+            Array containing the filter. Each line of the sinogram will be filtered with
             this filter. It has to be the Real-to-Complex Fourier Transform
             of some real filter, padded to 2*sinogram_width.
-        :param normalize: Whether to normalize the filter with pi/num_angles.
+        normalize: bool or float, optional
+            Whether to normalize (multiply) the filter with pi/num_angles.
         """
         if h_filt.size != self.sino_f_shape[-1]:
             raise ValueError(
@@ -106,11 +120,10 @@ class SinoFilter:
             )
         if not (np.iscomplexobj(h_filt)):
             print("Warning: expected a complex Fourier filter")
-        self.filter_f = h_filt
+        self.filter_f = h_filt.copy()
         if normalize:
             self.filter_f *= pi / self.n_angles
         self.filter_f = self.filter_f.astype(np.complex64)
-        self.d_filter_f[:] = self.filter_f[:]
 
     def _compute_filter(self, filter_name):
         self.filter_name = filter_name or "ram-lak"
@@ -128,52 +141,12 @@ class SinoFilter:
         filter_f = filter_f[: self.dwidth_padded // 2 + 1]  # R2C
         self.set_filter(filter_f, normalize=True)
 
-    def _init_kernels(self):
-        fname = get_cuda_srcfile("ElementOp.cu")
-        if self.ndim == 2:
-            kernel_name = "inplace_complex_mul_2Dby1D"
-            kernel_sig = "PPii"
-        else:
-            kernel_name = "inplace_complex_mul_3Dby1D"
-            kernel_sig = "PPiii"
-        self.mult_kernel = CudaKernel(kernel_name, filename=fname, signature=kernel_sig)
-        self.kern_args = (self.d_sino_f, self.d_filter_f)
-        self.kern_args += self.d_sino_f.shape[::-1]
-        self._pad_edges_kernel = CudaKernel(
-            "padding_edge", filename=get_cuda_srcfile("padding.cu"), signature="Piiiiiiii"
-        )
-        self._pad_block = (32, 32, 1)
-        self._pad_grid = tuple([updiv(n, p) for n, p in zip(self.sino_padded_shape[::-1], self._pad_block)])
-
     def _check_array(self, arr):
         if arr.dtype != np.float32:
             raise ValueError("Expected data type = numpy.float32")
         if arr.shape != self.sino_shape:
             raise ValueError("Expected sinogram shape %s, got %s" % (self.sino_shape, arr.shape))
 
-    def _pad_sino(self, sino):
-        if self.padding_mode == "edges":
-            self.d_sino_padded[:, : self.dwidth] = sino[:]
-            self._pad_edges_kernel(
-                self.d_sino_padded,
-                self.dwidth,
-                self.n_angles,
-                self.dwidth_padded,
-                self.n_angles,
-                self.pad_left,
-                self.pad_right,
-                0,
-                0,
-                grid=self._pad_grid,
-                block=self._pad_block,
-            )
-        else:  # zeros
-            self.d_sino_padded.fill(0)
-            if self.ndim == 2:
-                self.d_sino_padded[:, : self.dwidth] = sino[:]
-            else:
-                self.d_sino_padded[:, :, : self.dwidth] = sino[:]
-
     def filter_sino(self, sino, output=None, no_output=False):
         """
         Perform the sinogram siltering.
@@ -189,31 +162,59 @@ class SinoFilter:
             in self.d_sino_padded.
         """
         self._check_array(sino)
-        # copy2d/copy3d
-        self._pad_sino(sino)
-        # FFT
-        self.fft.fft(self.d_sino_padded, output=self.d_sino_f)
-
-        # multiply padded sinogram with filter in the Fourier domain
-        self.mult_kernel(*self.kern_args)  # TODO tune block size ?
-
-        # iFFT
-        self.fft.ifft(self.d_sino_f, output=self.d_sino_padded)
-
-        # return
-        if no_output:
-            return self.d_sino_padded
+        sino_padded = np.pad(
+            sino, ((0, 0), (0, self.dwidth_padded - self.dwidth)), mode=self.padding_mode
+        )  # pad with a FFT-friendly layout
+        sino_padded_f = rfft(sino_padded, axis=1, workers=get_num_threads(self.extra_options["fft_threads"]))
+        sino_padded_f *= self.filter_f
+        sino_filtered = irfft(sino_padded_f, axis=1, workers=get_num_threads(self.extra_options["fft_threads"]))
         if output is None:
             res = np.zeros(self.sino_shape, dtype=np.float32)
-            # can't do memcpy2d D->H ? (self.d_sino_padded[:, w]) I have to get()
-            sino_ref = self.d_sino_padded.get()
         else:
             res = output
-            sino_ref = self.d_sino_padded
         if self.ndim == 2:
-            res[:] = sino_ref[:, : self.dwidth]
+            res[:] = sino_filtered[:, : self.dwidth]  # pylint: disable=E1126 # ?!
         else:
-            res[:] = sino_ref[:, :, : self.dwidth]
+            res[:] = sino_filtered[:, :, : self.dwidth]  # pylint: disable=E1126 # ?!
         return res
 
     __call__ = filter_sino
+
+
+def filter_sinogram(
+    sinogram,
+    padded_width,
+    filter_name="ramlak",
+    padding_mode="constant",
+    normalize=True,
+    filter_cutoff=1.0,
+    **padding_kwargs,
+):
+    """
+    Simple function to filter sinogram.
+
+    Parameters
+    ----------
+    sinogram: numpy.ndarray
+        Sinogram, two dimensional array with shape (n_angles, sino_width)
+    padded_width: int
+        Width to use for padding. Must be greater than sinogram width (i.e than sinogram.shape[-1])
+    filter_name: str, optional
+        Which filter to use. Default is ramlak (roughly equivalent to abs(nu) in frequency domain)
+    padding_mode: str, optional
+        Which padding mode to use. Default is zero-padding.
+    normalize: bool, optional
+        Whether to multiply the filtered sinogram with pi/n_angles
+    filter_cutoff: float, optional
+        frequency cutoff for filter
+    """
+    n_angles, width = sinogram.shape
+    sinogram_padded = np.pad(sinogram, ((0, 0), (0, padded_width - width)), mode=padding_mode, **padding_kwargs)
+    fourier_filter = compute_fourier_filter(padded_width, filter_name, cutoff=filter_cutoff)
+    if normalize:
+        fourier_filter *= np.pi / n_angles
+    fourier_filter = fourier_filter[: padded_width // 2 + 1]  # R2C
+    sino_f = rfft(sinogram_padded, axis=1)
+    sino_f *= fourier_filter
+    sino_filtered = irfft(sino_f, axis=1)[:, :width]  # pylint: disable=E1126 # ?!
+    return sino_filtered
