@@ -1,7 +1,4 @@
 import numpy as np
-import pycuda.gpuarray as garray
-from ..cuda.kernel import CudaKernel
-from ..cuda.processing import CudaProcessing
 from .fbp import Backprojector
 from .reconstructor import Reconstructor
 
