@@ -1,21 +1,16 @@
 import os
+from bisect import bisect_left
 import numpy as np
 from silx.io import get_data
 from silx.io.url import DataUrl
 from tomoscan.esrf.scan.edfscan import EDFTomoScan
-from tomoscan.esrf.scan.hdf5scan import HDF5TomoScan
-
+from tomoscan.esrf.scan.nxtomoscan import NXtomoScan
 from ..utils import check_supported
 from ..io.utils import get_compacted_dataslices
 from .utils import is_hdf5_extension, get_values_from_file
 from .logger import LoggerOrPrint
 
-# Wait for next tomoscan release to ship "nexus_version"
-from packaging.version import parse as parse_version
-from tomoscan.version import version as tomoscan_version
-
-_tomoscan_has_nxversion = parse_version(tomoscan_version) > parse_version("0.6.0")
-#
+from ..pipeline.utils import nabu_env_settings
 
 
 class DatasetAnalyzer:
@@ -56,25 +51,29 @@ class DatasetAnalyzer:
             "output_dir": None,
             "exclude_projections": None,
             "hdf5_entry": None,
+            "nx_version": 1.0,
         }
-        if _tomoscan_has_nxversion:
-            advanced_options["nx_version"] = 1.0
         # --
         advanced_options.update(extra_options)
         self.extra_options = advanced_options
 
+    # pylint: disable=E1136
     def _get_excluded_projections(self):
+        self._ignore_projections_indices = None
+        self._need_rebuild_tomoscan_object_to_exclude_projections = False
         excluded_projs = self.extra_options["exclude_projections"]
         if excluded_projs is None:
             return
-        projs_idx = get_values_from_file(excluded_projs, any_size=True).astype(np.int32).tolist()
-        self.logger.info("Ignoring projections: %s" % (str(projs_idx)))
-        self.extra_options["exclude_projections"] = projs_idx
+        if excluded_projs["type"] == "indices":
+            projs_idx = get_values_from_file(excluded_projs["file"], any_size=True).astype(np.int32).tolist()
+            self._ignore_projections_indices = projs_idx
+        else:
+            self._need_rebuild_tomoscan_object_to_exclude_projections = True
 
     def _init_dataset_scan(self, **kwargs):
         if self._scanner is None:
             raise ValueError("Base class")
-        if self._scanner is HDF5TomoScan:
+        if self._scanner is NXtomoScan:
             if self.extra_options.get("hdf5_entry", None) is not None:
                 kwargs["entry"] = self.extra_options["hdf5_entry"]
             if self.extra_options.get("nx_version", None) is not None:
@@ -82,10 +81,50 @@ class DatasetAnalyzer:
         if self._scanner is EDFTomoScan:
             # Assume 1 frame per file (otherwise too long to open each file)
             kwargs["n_frames"] = 1
+
         self.dataset_scanner = self._scanner(  # pylint: disable=E1102
-            self.location, ignore_projections=self.extra_options["exclude_projections"], **kwargs
+            self.location, ignore_projections=self._ignore_projections_indices, **kwargs
         )
         self.projections = self.dataset_scanner.projections
+
+        # ---
+        if self._need_rebuild_tomoscan_object_to_exclude_projections:
+            # pylint: disable=E1136
+            exclude_projs = self.extra_options["exclude_projections"]
+            rot_angles_deg = np.rad2deg(self.rotation_angles)
+            self._rotation_angles = None  # prevent caching
+            # tomoscan only supports ignore_projections=<list of integers>
+            # However this is cumbersome to use, it's more convenient to use angular range or list of angles
+            # But having angles instead of indices implies to already have information on current scan angular range
+            ignore_projections_indices = []
+            if exclude_projs["type"] == "angular_range":
+                exclude_angle_min, exclude_angle_max = exclude_projs["range"]
+                projections_indices = np.array(sorted(self.dataset_scanner.projections.keys()))
+                for proj_idx, angle in zip(projections_indices, rot_angles_deg):
+                    if exclude_angle_min <= angle and angle <= exclude_angle_max:
+                        ignore_projections_indices.append(proj_idx)
+            elif exclude_projs["type"] == "angles":
+                excluded_angles = get_values_from_file(exclude_projs["file"], any_size=True).astype(np.float32).tolist()
+                for excluded_angle in excluded_angles:
+                    proj_idx = bisect_left(rot_angles_deg, excluded_angle)
+                    if proj_idx < rot_angles_deg.size:
+                        ignore_projections_indices.append(proj_idx)
+            # Rebuild the dataset_scanner instance
+            self._ignore_projections_indices = ignore_projections_indices
+            self.dataset_scanner = self._scanner(  # pylint: disable=E1102
+                self.location, ignore_projections=self._ignore_projections_indices, **kwargs
+            )
+        # ---
+        if self._ignore_projections_indices is not None:
+            self.logger.info("Excluding projections: %s" % str(self._ignore_projections_indices))
+
+        if nabu_env_settings.skip_tomoscan_checks:
+            self.logger.warning(
+                " WARNING: according to nabu_env_settings.skip_tomoscan_checks, skipping virtual layout integrity check of tomoscan which is time consuming"
+            )
+            self.dataset_scanner.set_check_behavior(run_check=False, raise_error=False)
+
+        self.projections = self.dataset_scanner.projections
         self.flats = self.dataset_scanner.flats
         self.darks = self.dataset_scanner.darks
         self.n_angles = len(self.dataset_scanner.projections)
@@ -106,7 +145,8 @@ class DatasetAnalyzer:
         self._energy = None
         self._pixel_size = None
         self._distance = None
-        self.flats_srcurrent = None
+        self._flats_srcurrent = None
+        self._projections_srcurrent = None
 
     @property
     def energy(self):
@@ -183,20 +223,43 @@ class DatasetAnalyzer:
     def detector_tilt(self, tilt):
         self._detector_tilt = tilt
 
+    def _get_srcurrent(self, indices):
+        srcurrent = self.dataset_scanner.electric_current
+        if srcurrent is None or len(srcurrent) == 0:
+            return None
+        srcurrent_all = np.array(srcurrent)
+        if np.any(indices >= len(srcurrent_all)):
+            self.logger.error("Something wrong with SRCurrent: not enough values!")
+            return None
+        return srcurrent_all[indices].astype("f")
+
     @property
     def projections_srcurrent(self):
         """
         Return the synchrotron electric current for each projection.
         """
-        srcurrent = self.dataset_scanner.electric_current
-        if srcurrent is None or len(srcurrent) == 0:
-            return None
-        srcurrent_all = np.array(srcurrent)
-        projections_indices = np.array(sorted(self.projections.keys()))
-        if np.any(projections_indices >= len(srcurrent_all)):
-            self.logger.error("Something wrong with SRCurrent: not enough values!")
-            return None
-        return srcurrent_all[projections_indices].astype("f")
+        if self._projections_srcurrent is None:
+            projections_indices = np.array(sorted(self.projections.keys()))
+            self._projections_srcurrent = self._get_srcurrent(projections_indices)
+        return self._projections_srcurrent
+
+    @projections_srcurrent.setter
+    def projections_srcurrent(self, val):
+        self._projections_srcurrent = val
+
+    @property
+    def flats_srcurrent(self):
+        """
+        Return the synchrotron electric current for each flat image.
+        """
+        if self._flats_srcurrent is None:
+            flats_indices = np.array(sorted(self.flats.keys()))
+            self._flats_srcurrent = self._get_srcurrent(flats_indices)
+        return self._flats_srcurrent
+
+    @flats_srcurrent.setter
+    def flats_srcurrent(self, val):
+        self._flats_srcurrent = val
 
     def check_defined_attribute(self, name, error_msg=None):
         """
@@ -252,7 +315,7 @@ class EDFDatasetAnalyzer(DatasetAnalyzer):
             scan_range = self.dataset_scanner.scan_range
             if scan_range is not None:
                 fullturn = abs(scan_range - 360) < abs(scan_range - 180)
-                angles = np.linspace(0, scan_range, num=self.dataset_scanner.tomo_n, endpoint=fullturn, dtype="f")
+                angles = np.linspace(0, scan_range, num=len(self.projections), endpoint=fullturn, dtype="f")
                 self._rotation_angles = np.deg2rad(angles)
         return self._rotation_angles
 
@@ -262,7 +325,7 @@ class HDF5DatasetAnalyzer(DatasetAnalyzer):
     HDF5 dataset analyzer
     """
 
-    _scanner = HDF5TomoScan
+    _scanner = NXtomoScan
     kind = "hdf5"
 
     @property
@@ -361,14 +424,17 @@ def analyze_dataset(dataset_path, extra_options=None, logger=None):
     return dataset_structure
 
 
-def get_0_180_radios(dataset_info, return_indices=False):
+def get_radio_pair(dataset_info, radio_angles: tuple, return_indices=False):
     """
-    Get the radios at 0 degres and 180 degrees.
+    Get closest radios at radio_angles[0] and radio_angles[1]
+    angles must be in angles
 
     Parameters
     ----------
     dataset_info: `DatasetAnalyzer` instance
         Data structure with the dataset information
+    radio_angles: tuple
+        tuple of two elements: angles (in radian) to get
     return_indices: bool, optional
         Whether to return radios indices along with the radios array.
 
@@ -378,14 +444,20 @@ def get_0_180_radios(dataset_info, return_indices=False):
         If return_indices is True, return a tuple (radios, indices).
         Otherwise, return an array with the radios.
     """
+    if not (isinstance(radio_angles, tuple) and len(radio_angles) == 2):
+        raise TypeError("radio_angles should be a tuple of two elements.")
+    if not isinstance(radio_angles[0], (np.floating, float)) or not isinstance(radio_angles[1], (np.floating, float)):
+        raise TypeError(
+            f"radio_angles should be float. Get {type(radio_angles[0])} and {type(radio_angles[1])} instead"
+        )
+
     radios_indices = []
     radios_indices = sorted(dataset_info.projections.keys())
     angles = dataset_info.rotation_angles
     angles = angles - angles.min()
-    i_0 = np.argmin(np.abs(angles))
-    i_180 = np.argmin(np.abs(angles - np.pi))
-    _min_indices = [i_0, i_180]
-    radios_indices = [radios_indices[i_0], radios_indices[i_180]]
+    i_radio_1 = np.argmin(np.abs(angles - radio_angles[0]))
+    i_radio_2 = np.argmin(np.abs(angles - radio_angles[1]))
+    radios_indices = [radios_indices[i_radio_1], radios_indices[i_radio_2]]
     n_radios = 2
     radios = np.zeros((n_radios,) + dataset_info.radio_dims[::-1], "f")
     for i in range(n_radios):
