@@ -4,7 +4,7 @@ from shutil import rmtree
 import pytest
 import numpy as np
 from silx.io import get_data
-from tomoscan.esrf.scan.hdf5scan import ImageKey
+from nxtomo.nxobject.nxdetector import ImageKey
 from nabu.testutils import generate_nx_dataset
 from nabu.resources.nxflatfield import update_dataset_info_flats_darks
 from nabu.resources.dataset_analyzer import HDF5DatasetAnalyzer
