@@ -1,8 +1,8 @@
 from itertools import product
-import posixpath
 import tarfile
 import os
 import numpy as np
+from scipy.signal.windows import gaussian
 from silx.resources import ExternalResources
 from silx.io.dictdump import dicttoh5, nxtodict, dicttonx
 from silx.io.url import DataUrl
@@ -59,6 +59,24 @@ def get_data(*dataset_path):
     return np.load(dataset_downloaded_path)
 
 
+def get_array_of_given_shape(img, shape, dtype):
+    """
+    From a given image, returns an array of the wanted shape and dtype.
+    """
+
+    # Tile image until it's big enough.
+    # "fun" fact: using any(blabla) crashes but using any([blabla]) does not, because of variables re-evaluation
+    while any([i_dim <= s_dim for i_dim, s_dim in zip(img.shape, shape)]):
+        img = np.tile(img, (2, 2))
+    if len(shape) == 1:
+        arr = img[: shape[0], 0]
+    elif len(shape) == 2:
+        arr = img[: shape[0], : shape[1]]
+    else:
+        arr = np.tile(img, (shape[0], 1, 1))[: shape[0], : shape[1], : shape[2]]
+    return np.ascontiguousarray(np.squeeze(arr), dtype=dtype)
+
+
 def get_big_data(filename):
     if __big_testdata_dir__ is None:
         return None
@@ -136,6 +154,52 @@ def compare_arrays(arr1, arr2, tol, diff=None, absolute_value=True, percent=None
     return res
 
 
+def gaussian_apodization_window(shape, fwhm_ratio=0.7):
+    fwhm = fwhm_ratio * np.array(shape)
+    sigma = fwhm / 2.355
+    return np.outer(*[gaussian(n, s) for n, s in zip(shape, sigma)])
+
+
+def compare_shifted_images(img1, img2, fwhm_ratio=0.7, return_upper_bound=False):
+    """
+    Compare two images that are slightly shifted from one another.
+    Typically, tomography reconstruction wight slightly different CoR.
+    Each image is Fourier-transformed, and the modulus is taken to get rid of the shift between the images.
+    An apodization is done to filter the high frequencies that are usually less relevant.
+
+    Parameters
+    ----------
+    img1: numpy.ndarray
+        First image
+    img2: numpy.ndarray
+        Second image
+    fwhm_ratio: float, optional
+        Ratio defining the apodization in the frequency domain.
+        A small value (eg. 0.2) means that essentually only the low frequencies will be compared.
+        A value of 1.0 means no apodization
+    return_upper_bound: bool, optional
+        Whether to return a (coarse) upper bound of the comparison metric
+
+    Notes
+    -----
+    This function roughly computes
+        |phi(F(img1)) - phi(F(img2))|
+    where F is the absolute value of the Fourier transform, and phi some shrinking function (here arcsinh).
+    """
+
+    def _fourier_transform(img):
+        return np.arcsinh(np.abs(np.fft.fftshift(np.fft.fft2(img))))
+
+    diff = _fourier_transform(img1) - _fourier_transform(img2)
+    diff *= gaussian_apodization_window(img1.shape, fwhm_ratio=fwhm_ratio)
+    res = np.abs(diff).max()
+    if return_upper_bound:
+        data_range = np.max(np.abs(img1))
+        return res, np.arcsinh(np.prod(img1.shape) * data_range)
+    else:
+        return res
+
+
 class SimpleHDF5TomoScanMock:
     def __init__(self, image_key):
         self._image_key = image_key
