@@ -1,12 +1,11 @@
+from functools import partial
 import os
 import sys
-from typing import Union
-import typing
+from functools import partial, lru_cache
+from itertools import product
 import warnings
 from time import time
 import posixpath
-from itertools import product
-from functools import lru_cache
 import numpy as np
 
 
@@ -62,6 +61,12 @@ def get_cuda_srcfile(filename):
     return os.path.join(cuda_src_folder, filename)
 
 
+def get_opencl_srcfile(filename):
+    src_relpath = os.path.join("opencl", "src")
+    src_folder = get_folder_path(foldername=src_relpath)
+    return os.path.join(src_folder, filename)
+
+
 def get_resource_file(filename, subfolder=None):
     subfolder = subfolder or []
     relpath = os.path.join("resources", *subfolder)
@@ -70,7 +75,11 @@ def get_resource_file(filename, subfolder=None):
 
 
 def get_available_threads():
-    return len(os.sched_getaffinity(0))
+    try:
+        n_threads = len(os.sched_getaffinity(0))
+    except AttributeError:
+        n_threads = int(os.environ.get("SLURM_CPUS_PER_TASK", os.cpu_count()))
+    return n_threads
 
 
 def is_writeable(location):
@@ -517,13 +526,16 @@ def filter_str_def(elmt):
         return None
     assert isinstance(elmt, str)
     elmt = elmt.lstrip(" ").rstrip(" ")
+    elmt = elmt.lstrip("'").lstrip('"')
+    elmt = elmt.rstrip("'").rstrip('"')
+    elmt = elmt.lstrip(" ").rstrip(" ")
     for character in ("'", '"'):
         if elmt.startswith(character) and elmt.endswith(character):
             elmt = elmt[1:-1]
     return elmt
 
 
-def convert_str_to_tuple(input_str: str, none_if_empty: bool = False) -> Union[None, tuple]:
+def convert_str_to_tuple(input_str: str, none_if_empty: bool = False):
     """
     :param str input_str: string to convert
     :param bool none_if_empty: if true and the conversion is an empty tuple
@@ -558,7 +570,7 @@ class Progress:
         self._name = name
         self.reset()
 
-    def reset(self, max_: typing.Union[None, int] = None) -> None:
+    def reset(self, max_=None):
         """
         reset the advancement to n and max advancement to max_
         :param int max_:
@@ -609,13 +621,26 @@ def concatenate_dict(dict_1, dict_2) -> dict:
     """update dict which has dict as values. And we want concatenate those values to"""
     res = dict_1.copy()
     for key in dict_2:
-        if key in dict_1 and key in dict_2:
+        if key in dict_1:
             res[key].update(dict_2[key])
         else:
             res[key] = dict_2[key]
     return res
 
 
+class BaseClassError:
+    def __init__(self, *args, **kwargs):
+        raise ValueError("Base class")
+
+
+def MissingComponentError(msg):
+    class MissingComponentCls:
+        def __init__(self, *args, **kwargs):
+            raise RuntimeError(msg)
+
+    return MissingComponentCls
+
+
 # ------------------------------------------------------------------------------
 # ------------------------ Image (move elsewhere ?) ----------------------------
 # ------------------------------------------------------------------------------
@@ -635,6 +660,8 @@ def generate_coords(img_shp, center=None):
 
 def clip_circle(img, center=None, radius=None):
     R, C = generate_coords(img.shape, center)
+    if radius is None:
+        radius = R.shape[-1] // 2
     M = R**2 + C**2
     res = np.zeros_like(img)
     res[M < radius**2] = img[M < radius**2]
@@ -760,3 +787,37 @@ def deprecation_warning(message, do_print=True, func_name=None):
     if do_print:
         print("Deprecation warning: %s%s" % (func_name_msg, message))
     _warnings[func_name] = 1
+
+
+def _docstring(dest, origin):
+    """Implementation of docstring decorator.
+
+    It patches dest.__doc__.
+    """
+    if not isinstance(dest, type) and isinstance(origin, type):
+        # func is not a class, but origin is, get the method with the same name
+        try:
+            origin = getattr(origin, dest.__name__)
+        except AttributeError:
+            raise ValueError("origin class has no %s method" % dest.__name__)
+
+    dest.__doc__ = origin.__doc__
+    return dest
+
+
+def docstring(origin):
+    """Decorator to initialize the docstring from another source.
+
+    This is useful to duplicate a docstring for inheritance and composition.
+
+    If origin is a method or a function, it copies its docstring.
+    If origin is a class, the docstring is copied from the method
+    of that class which has the same name as the method/function
+    being decorated.
+
+    :param origin:
+        The method, function or class from which to get the docstring
+    :raises ValueError:
+        If the origin class has not method n case the
+    """
+    return partial(_docstring, origin=origin)
